#!/usr/bin/python

##############################################
##                                          ##
## WVR scaling main run file                ##
## Developed by Luke T. Maud                ##
## Allegro, ALMA Arc Node, Leiden           ##
##                                          ##
##                                          ##
## Version:                                 ##
## V1.0 - 18/05/2017 - initial release      ##
## V1.1 - 13/06/2017 - slight plot edits    ##
##                     pre paper acceptance ##
##                                          ##
## NB: version edits refer to any changes   ##
## made in this code or supporting modules  ##
##############################################

#################################################################
##                                                             ##
## This is the only run file required for ALMA users to run    ##
## in order to create all files required to analyse any        ##
## ALMA data in order to calculate the best WVR scaling factor ##
##                                                             ##
## THIS SCRIPT MUST BE SAVED IN THE SAME DIRECTORY AS THE      ##
## ALMA DATA DIRECTORIES                                       ##
##                                                             ##
## For more details of the script options please see           ##
## the comments below and the README                           ## 
##                                                             ##
## For more details on the scaling analysis please see:        ##
## Maud et al. 2017 A&A XXX XXX XXX                            ##
##                                                             ##
#################################################################



#####################################
##                                 ##
##    USER input paramters         ##
##                                 ##
#####################################

##  User MUST set this PATH to the directory where the scaling
##  module is unpacked and scaling scripts reside if not usr/lib
## e.g. sys.path.append('/lustre2/allegro/maud/customWVRscaling') 
import sys

#sys.path.append('pathtomodules')  

##########################
## Required imports
import sys
from glob import glob
import numpy as np
import wvrScaleAnalysis
import wvrScalePlotting
import wvrScaleImaging
##########################

## The code will automatically run the scaling analysis selected
## on ALL asdm files in the RAW directory. IF only one asdm of 
## many is the intended target, then uncomment the following 
## and input the specific asdm file name
## e.g. asdms =  ['raw/uid___A002_X91bdc6_X9dd']

#asdms= ['raw/filename']  

## Options are True or False - DEFAULT = True
## True takes longer to run but makes full fits 
## files with Two-Point-Deviation analysis 
fullReduct = True 

## Options are True or False - DEFAULT = True
## True is ONLY effective if the fullReduct option is also True 
makeTPDplots = True 

## Options are [''] (a blank string) or an input with the
## reference antenna e.g. ['DA51']
## A blank string runs the 'all baseline' analysis (Typical run time 30-60 miuntes)
## Specifying a reference antenna runs the 'ref antenna only' 
## analysis (Typical run time 3-10 minutes)
## IF you have more than ONE asdm file for analysis and you want 
## to use the reference antenna analysis you MUST 
## specify all antennas e.g. ['DA51', 'DA51'] - for two files
## or specify both as blank e.g. ['',''] - for two files
refant = [''] 

## IMPORTANT NOTE
## As the module remakes the data files
## any CASA based flags are NOT used - therefore
## it is very important to note ALL flags below

## Options are [''] (blank string) for no antennas to exclude
## or an input string e.g. ['DV21,DV25'] to exclude both 
## these antennas in the analysis
## IF you have more than ONE asdm file for analysis you must specify 
## antennas to exclude for ALL files using comma separated strings
## e.g. ['DV21','DV21,DV25,DA43']  - to flag DV21 for asdm 1, 
## and DV21,DV25,DA43 for asdm 2 or ensure there are comma 
## separated blank strings, e.g. ['','']
## Note these are NOT permanant CASA flags to the data, they 
## are only excluded for the WVR analysis BUT do really flag 
## out in the ms files produced for the Bandpass images
flagant = [''] 

## Options are [''] (blank string) to not exclude any baselines
## or an input string e.g. ['DA55-DA51'] to exclue this baseline
## For multiple baselines to flag, use comma separation WITHIN the string
## e.g. 'DA55-DA51,DA56-DA53'  - will exclude these two baselines
## IF you have more than ONE asdm file for analysis you must specify 
## the baselines for ALL files using comma separated strings
## e.g. 'DA55-DA51','DA55-DA53' this excludes DA55-DA51 in asdm 1 and DA55-DA53 in asdm 2
## or ensure there are comma separated blank strings, e.g. ['','']
## Note these are NOT permanant CASA flags to the data, they are only excluded for the WVR analysis
flagbl = ['']  

## Options are True or False - DEFAULT = False
## this will plot the details of the WVR scaling using
## the 12 second TPD timescale - this is also used for the coherence estimate
## IF the phase improvement ratio is `better' for the 12 second timescale
## it would be useful to re-run this script with use12=True
use12=  False

## Options are True or False - DEFAULT = False
## this option will re-extract the ms files for the bandpass source and image 
## them with standard WVR (scale=1.0) and scaled WVR (found in the analysis)
## Note this will take some time as Tsys, WVR calibrations, Bandpass frequency dependence 
## and the single phase offset are all calculated and applied before imaging takes place
imageBP = True 

## Option to set the flux and spectral index of the bandpass source
## for the calibration at a given reference frequency 
## - if these are set to None (default)
## then the flux is normalised to 1.0 Jy and the spectral index set to 0.0
## and the frequency that of the band (default is empty string so the code will find it)
## however it is recommended to set it IF you set the flux and spectral index manually
fluxInput =  1.0
specInput =  0.0
freqInput = ''


## Options are True or False - DEFAULT = False
## If True and the data exist it will re-run the code and re-make all files
## If False it will use the existing data - takes less time if run once already
## (do not need to recreate all ms files and fits files)
## Note the image analysis will ALWAYS overwrite the files
overwrite= False 



#####################################
##                                 ##
##     End PI input parameters     ##
##                                 ##
#####################################




###############
## MAIN CODE ##
###############

algorithm = 'calc2pt'  # this is a fixed to call the algorithm


try:
    len(asdms)
except NameError:
    asdms = glob('raw/*')


try:     
    for asdm in asdms:
        print ' - %s' % asdm
        


    # create directories if necessary - imaging derectory ONLY created later if requested
    if not os.path.exists('fits'):
	os.system('mkdir fits')
    if not os.path.exists('ms'):
	os.system('mkdir ms')
    if not os.path.exists('plots'):
	os.system('mkdir plots')

    # loop over the asdms
    for noasdm,asdm in enumerate(asdms):

	WVRS_values = []  
        warn_inputs=False
	print ''
	print ''
	print " Working on ..."
	print " == %s == " % asdm.split('/')[-1]
        print ''
        print " Please ensure all antennas or baselines to be flagged "
        print " have been listed as CASA flags are NOT used from "
        print " the delivered reduction scripts "

	refant_use=refant[noasdm]
	flagant_use=flagant[noasdm]
	flagbl_use=flagbl[noasdm]

        ## Test to check input are correct in length 
	if len(refant) < len(asdms):
		warn_inputs=True
		print ''
		print ' Warning: refant length does not match number of ASDMs input'
		print ' Defaulting all to full baseline analysis'
		print ''
		refant_use=''
	if len(flagant) < len(asdms):
		warn_inputs=True
		print ''
		print ' Warning: flagant length does not match number of ASDMs input'
		print ' No antennas will be excluded'
		print ''
		flagant_use=''
	if len(flagbl) < len(asdms):
		warn_inputs=True
		print ''
		print ' Warning: flagbl length does not match number of ASDMs input'
		print ' No baselines will be exculed'
		print ''
		flagbl_use=''



        # Generate the files
        filenames= wvrScaleAnalysis.WVRS_generate_files(asdm, overwrite=overwrite, outputdir='ms')
	filenames['pwv'] = filenames['vis'].replace('.ms', '.pwv')
        if ( not os.path.exists(filenames['pwv']) ) or ( os.path.exists(filenames['pwv']) and overwrite ):
	     os.system('ln -s %s %s' % (os.path.basename(filenames['vis']), filenames['pwv']))

    
        prefix = '%s' % asdm.split('/')[-1]
                 
        if fullReduct:
            # Now get the useful metadata
            print ""
            print " ** Generating Metadata"
            M = wvrScaleAnalysis.WVRS_metadata(filenames['vis']) 
            print " ** Done Metadata "


            for key in ['vis','wvr','viswvr']:
                if os.path.exists(filenames[key]):
                    fitsprefix='fits/%s.%s' % (prefix,key)
                    
                    
                    print ""
                    print " ** Calculating the full "
                    print "    Two-Point-Deviations of the standard data"
                    
                    wvrScaleAnalysis.WVRS_two_point_deviation_func(filenames[key], prefix=fitsprefix, meta=M, unwrap=True,  algorithm=algorithm, wvr_outlier=False, single=True,Flags=True)

                    if makeTPDplots:
                        print ""
                        print " ** Producing the TDP plots and phases streams "
                        print "    for ALL baselines .... this will take time "
                        wvrScalePlotting.WVRS_plotTPDS('%s.fits' % fitsprefix, figdir='plots/%s/%s' % (prefix,key), overwrite=True, eps=False, TeX=False)

            print ""
            print " ** Doing the WVR scale factor analysis and plots "

            WVRS_values.append(wvrScalePlotting.WVRScaleFactor(vis_run=prefix, refant=refant_use,badant=flagant_use,badbl=flagbl_use,use12=use12))
            
            
          

        else:
            # Short version
            print ""
            print " Opted for quicker analysis which will "
            print "  only include WVR scaling plots "

            print ""
            print " ** Generating Short Metadata"
            M = wvrScaleAnalysis.WVRS_metashort(filenames['vis']) 
            print " ** Done Short Metadata "

            
            for key in ['vis','wvr','viswvr']:  
                if os.path.exists(filenames[key]):
                    fitsprefix='fits/%s.%s' % (prefix,key)
                    
                    if os.path.exists(fitsprefix+'.fits') and overwrite or (not os.path.exists(fitsprefix+'.fits')):
                        print ""
                        print " ** Filling the fits files "
                        print "   required for WVR scaling ONLY"
                        wvrScaleAnalysis.WVRS_two_point_deviation_func(filenames[key], prefix=fitsprefix, meta=M, unwrap=True, wvr_outlier=False, single=True, fitsonly=True,Flags=True)
                    else:
                        print " ** Fits files already exist "

            print ""
            print " ** Doing the WVR scale factor analysis and plots "

            WVRS_values.append(wvrScalePlotting.WVRScaleFactor(vis_run=prefix, refant=refant_use,badant=flagant_use,badbl=flagbl_use,use12=use12))


        # print the output to a log file 
        timerunning = time.strftime("%Y%m%d-%H%M%S")
        if not use12:
            outputfile = open('WVRscalingResults_6sec_'+asdm.split('/')[-1]+'_'+timerunning+'.txt','w')

            outputfile.write('Coherence values use the 6 second TPD noise statistic'+ '\n')
        else:
            outputfile = open('WVRscalingResults_12sec_'+asdm.split('/')[-1]+'_'+timerunning+'.txt','w')

            outputfile.write('Coherence values use the 12 second TPD noise statistic'+ '\n')
        for prti in ['data','6sec_scalefactor','12sec_scalefactor','32sec_scalefactor','64sec_scalefactor','phaseImp_ratio@6','phaseImp_ratio@12','CohImp30sec','CohImp60sec','CohImp90sec','CohImp120sec','CohImpAll','CohRawAve60','CohWVRAve60','CohScaledAve60','CohRawAveAll','CohWVRAveAll','CohScaledAveAll']: # the keywords
            outputfile.write(prti+' , '+str(WVRS_values[noasdm][prti])+'\n')
    
        if imageBP:
            print ''
            print ' Running the Imaging Analysis '
            print ' NOTE: this may take some time '
            print ''
            
            if use12:
                scale_use = WVRS_values[noasdm]['12sec_scalefactor']
            else:
                scale_use = WVRS_values[noasdm]['6sec_scalefactor']

            # data analysis and reduction
            imagefilenames = wvrScaleImaging.WVRS_generate_files_forBPimage(asdm, overwrite=False, scale=scale_use, flag_ants=flagant_use, meta_use=M, refant_use=refant_use, fluxSet=fluxInput, specSet=specInput, freqSet = freqInput)
                
          
            # image and get the stats 
            imageBpStats = []
            for key in ['wvrcal','wvrsclcal']:
                print ''
                print ' * Making image and getting stats for %s'%imagefilenames[key]
                imageBpStats.append(wvrScaleImaging.makeBPimages(imagefilenames[key], overwrite=False,freq_obs=M['ref_freq'],band_obs=M['band'], max_bl=np.max(M['bl']['uv']), meta_use=M, fluxSet=fluxInput))

            for prtimg in ['keys']: # the keywords
                outputfile.write(prti+' , '+str(WVRS_values[noasdm][prti])+'\n')
        
            # make a png image 
            wvrScaleImaging.makeBPpng(asdm, statsImg=imageBpStats, fluxSet=fluxInput)

            for img_no,img_type in enumerate(['Std_WVR:','Scaled_WVR:']):
                outputfile.write(img_type+'\n') 
                for prtimg in ['peak','rms','coh','snr']: # the keywords
                    outputfile.write(prtimg+' , '+str(imageBpStats[img_no][prtimg])+'\n') 

        outputfile.close()


    if warn_inputs:
	    print ''
	    print ' Note that some of the inputs were not entered '
	    print ' correctly for either the refant, flagant, flagbl '
	    print ' as multiple ASDM files were found but options '
	    print ' for were not specified for all'
	    print ' See the warning, inputs and/or README_wvrScaling for help'

  
    

except:
    print ''
    print ' There is an inconsitency in the files or options'
    print ' check there is an ASDM file in the raw directory'
    print ' check there is a refant specified for refant only analysis'
    print ' check string lists are supplied for options where required'
    print ' check all options True/False'



#####################
## End of the code ##
#####################

