## this is the set of analysis codes for WVR scaling
## Developed at Allegro ALMA Arc node Leiden 

from wvrScaleTools import *
from wvrScaleImportFuncs import *
import warnings 

def WVRS_generate_files(asdm, verbose = True, overwrite=False, intent=None, outputdir=None, scans_use=False):
    ''' Generates all the files for the WVR analysis, namely:
    * measurement set (``asdm.ms``)
    * measurement set with no data (``asdm.unity.ms``)
    * wvr calibration table (``asdm.wvr``)
       
    :param asdm: ASDM filename
    :type asdm: str
    :keyword verbose: verbose
    :type verbose: boolean
    :keyword overwrite: overwrite existing files
    :type overwrite: boolean
    :keyword intent: observing intent of the scans to analyse
    :type intent: str
    :keyword outputdir: copy the created files into this directory
    :type outputdir: str
    :param scans_use: to specify scans to read in for field/source e.g. '0:3'
    :type scans_use: string
    '''
    
    summary={}
    
    if not os.path.exists(outputdir):
        os.system('mkdir %s' % outputdir)
    
    # filenames
    if outputdir is None:
        vis  = '%s.ms' % asdm
        vis1 = '%s.unity.ms' % asdm
        wvr  = '%s.ms.wvr' % asdm
    else:
        vis = '%s/%s.ms' % (outputdir, asdm.split('/')[-1])
        vis1 = '%s/%s.unity.ms' % (outputdir, asdm.split('/')[-1])
        wvr = '%s/%s.ms.wvr' % (outputdir, asdm.split('/')[-1])
    
    summary['vis']=vis
    summary['viswvr']=vis1
    summary['wvr']=wvr
    
    if verbose: print " * Reading in ASDM"
    
    if ( not os.path.exists(vis) ) or ( os.path.exists(vis) and overwrite):
        if intent is None:
            if verbose: print " Reading in BANDPASS of %s" % asdm
            check = asdm_importToCASA(asdm, vis=vis, intent='BANDPASS', verbose=verbose, scans_use=scans_use)
            intent = 'BANDPASS'
            if check is False:
                if verbose: print " no BANDPASS found!"
                if verbose: print " Reading in PHASE of %s" % asdm
                check = asdm_importToCASA(asdm, vis=vis, intent='PHASE', verbose=verbose, scans_use=scans_use)
                intent = 'PHASE'
        else:
            check = asdm_importToCASA(asdm, vis=vis, intent=intent, verbose=verbose, scans_use=scans_use)
            intent = 'FLUX'
    # resetting the asdm and dataset_type
    else:
        print "   Nothing to do. MS is already there"
    
    # if we have a MS, we perform the WVR calibration and apply it
    if verbose: print " * WVR correction"
    
    # check if the measurement set is already WVR calibrated
    is_wvr_calibrated = WVRcheck(vis)
    #    print "the calibration state is: ", calibrated
    if (is_wvr_calibrated):
        print "   Nothing to do. MS has already WVR corrections applied"
    else:
        # if do_log: logtime.write("Starting WVR calibration")
        WVRcalib(vis, wvr, overwrite=overwrite)
    
    # create the unity-file and apply the calibration to it
    if ( not os.path.exists(vis1) ) or ( os.path.exists(vis1) and overwrite):
        print "create the unity file"
        os.system('cp -r %s %s' % (vis,vis1))
        ms.open(vis1, nomodify=False)
        spws = ms.getspectralwindowinfo()
        ms.close()
        for i in spws:
            #print ''
            #print i 
            try:
                #print ' runing'
                ms.open(vis1, nomodify=False)
                ms.selectinit(datadescid=np.int(i))
                data=ms.getdata('DATA')
                data['data']=data['data']*0.+1.
                ms.putdata(data)
                ms.close()
            except RuntimeError:   ## the narrow SPW average is causing an issue
                print 'issue - no unity in SPW', i
                
    
        WVRcalib(vis1, wvr, overwrite=False)
    
    return summary

def WVRS_metashort(vis, spw=None, verbose=False):
    ''' returns a dictionary with metadata from a MS - short version 
    
    :param vis: measurement set
    :type vis: str
    :keyword str spw: spectral window (default: last chan-averaged spw)
    :keyword boolean verbose: verbose
    :returns: dictionary with metadata
    :rtype: dict
    
    The returned dictionary includes:
    
    
    * ``date-obs``  - the time of the start of the observations
    * ``band``      - the ALMA Band
    * ``ref_freq``  - the reference frequency
    * ``spw``       - the spectral window ID of the reference frequency
    * ``pol``       - polarisations
    * ``ant``       - dictionary with antenna parameters
        * antenna name
        * X
        * Y
        * Z position
    * ``bl``        - dictionary with baseline parameters
        * name
        * median baseline length in UV
        * baseline length on the ENU plain (m)
        * baseline length in E-W direction
        * baseline length in N-S direction (up-down is omitted for now)
  
    * ``pwv``       - pwv (mm)
   
    
    '''
    
    # metadata
    if verbose:
        print "Entering metadata"
    
    summary = {}
    
    # checking the timerange
    tb.open(vis)
    t0 = tb.getcol('TIME')[0]
    tb.close()
    
    timequery=''
    summary['timequery']=timequery
    t0-=40587*86400. # convert to unix timestamp
    t0str=datetime.fromtimestamp(t0, tz=utc).strftime('%Y/%m/%d/%H:%M:%S')
    d0str=datetime.fromtimestamp(t0, tz=utc).strftime('%Y-%m-%d')
    summary['date-obs']=t0str

    # get the spectral window  - another different way....
    tb.open('%s/SPECTRAL_WINDOW' % vis)
    if spw is None:
        
        spw=getLastSpwID_ms(vis)
    ref_freq = tb.getcol('REF_FREQUENCY')[spw]
    band     = getband(ref_freq)
    tb.close()
    
    summary['band']=band
    summary['ref_freq']=ref_freq
    summary['spw']=spw
    
    # antenna
    # antenna positions
    x,y,z,antnames=antennaENU(vis)
    summary['ant']={'name': antnames, 'y': y, 'x': x, 'z': z}
    if len(summary['ant']['name']) > 45:
    
        for key_lp in ['name','x','y']:
            summary['ant'][key_lp]=summary['ant'][key_lp][0:45]

    nant=len(summary['ant']['name'])
    
    # these arrays contain...
    uv = []  # ... median bl-length in the UV plane (m)
    xy = []  # ... XY bl-length in the ENU plane (m)
    xbl = [] # ... dX coordinate of the antenna (m)
    ybl = [] # ... dY coordinate of the antenna (m)
    bl = []  # ... baseline name (ANT1-ANT2)
    
    tb.open(vis)
    for i in range(nant-1):
        for j in range(i+1, nant):
            tb1=tb.query('ANTENNA1==%i && ANTENNA2==%i%s' % (i,j,timequery))
            try:
                u,v,w=tb1.getcol('UVW')
            except:
                u,v,w=np.nan,np.nan,np.nan
            bl.append('%s-%s' % (antnames[i], antnames[j]))
            uv.append(np.median(np.hypot(u,v)))
            xy.append(np.hypot( x[i]-x[j], y[i]-y[j] ))
            xbl.append( x[i]-x[j] )
            ybl.append( y[i]-y[j] )
    tb.close()
    
    summary['bl']={'name': bl, 'uv': uv, 'xy': xy, 'dx': xbl, 'dy': ybl}

    # polarisations
    tb.open('%s/DATA_DESCRIPTION' % vis)
    pol_id=tb.getcol('POLARIZATION_ID')[spw]
    tb.close()
    
    tb.open('%s/POLARIZATION' % vis)
    pol=stokes(tb.getvarcol('CORR_TYPE')['r%i' % (1+pol_id)][:,0])
    tb.close()
    summary['pol']=pol

    
    # getting the pwv
    calwvr='%s/ASDM_CALWVR' % vis
    pwv = -1
    if os.path.exists(calwvr):
        tb.open(calwvr)
        water = tb.getcol('water')
        pwv = np.mean(water[np.isfinite(water)])*1000  # changed to mean - already average value PER antenna
        pwv_err = np.std((water[np.isfinite(water)])*1000)
        if np.isnan(pwv):
            pwv=-1
        tb.close()
    
       
    summary['pwv']=pwv
    summary['pwv_err']=pwv_err  # added

    
    if verbose:
        print "Finished Short metadata"
    
    return summary


def WVRS_metadata(vis, spw=None, timerange=None, verbose=False):
    ''' returns a dictionary with metadata from a MS.
    
    :param vis: measurement set
    :type vis: str
    :keyword str spw: spectral window (default: last chan-averaged spw)
    :keyword float timerange: array with [t_min, t_max] from the start of the
        first observation. So, timerange=[300, 600] will analyse the
        data that has been taken between 5 and 10 minutes after the start of the
        observations.
    :keyword boolean verbose: verbose
    :returns: dictionary with metadata
    :rtype: dict
    
    The returned dictionary includes:
    
    * ``timequery`` - the timequery for the extraction of MS
    * ``date-obs``  - the time of the start of the observations
    * ``band``      - the ALMA Band
    * ``ref_freq``  - the reference frequency
    * ``spw``       - the spectral window ID of the reference frequency
    * ``ant``       - dictionary with antenna parameters
        * antenna name
        * X
        * Y
        * Z position
    * ``bl``        - dictionary with baseline parameters
        * name
        * median baseline length in UV (m) 
        * baseline length on the ENU plain (m)
        * baseline length in E-W direction
        * baseline length in N-S direction (up-down is omitted for now)
    * ``pol``       - polarisations
    * ``pwv``       - pwv (mm)
    * ``weather``   - dictionary with weather data
    * ``field``     - dictionary with field data
        * name
        * right ascension
        * declination
    * ``scan``      - dictionary with azimuth-elevation info
        * scan number
        * mean time
        * mean altitude
        * mean azimuth
    '''
    
    # metadata
    if verbose:
        print "Entering metadata"
    
    summary = {}
    
    # checking the timerange
    tb.open(vis)
    t0 = tb.getcol('TIME')[0]
    tb.close()
    # string for the query
    if timerange is not None:
        timerange=np.array(timerange)+t0
        timequery=' && TIME >= {0} && TIME <= {1}'.format(timerange[0], timerange[1])
        t0=timerange[0]
    else:
        timequery=''
    summary['timequery']=timequery
    t0-=40587*86400. # convert to unix timestamp
    t0str=datetime.fromtimestamp(t0, tz=utc).strftime('%Y/%m/%d/%H:%M:%S')
    d0str=datetime.fromtimestamp(t0, tz=utc).strftime('%Y-%m-%d')
    summary['date-obs']=t0str

    # get the spectral window  - another different way....
    tb.open('%s/SPECTRAL_WINDOW' % vis)
    spw_names = tb.getcol('NAME')
    if spw is None:
        #num_chan = tb.getcol('NUM_CHAN')
        #spw=np.max([i for i,j in enumerate(num_chan) if j == 1])
        spw=getLastSpwID_ms(vis)
        #else:
            #spw=np.max([i for i,j in enumerate(spw_names) if 'CH_AVG' in j])
    ref_freq = tb.getcol('REF_FREQUENCY')[spw]
    band     = getband(ref_freq)
    tb.close()
    
    summary['band']=band
    summary['ref_freq']=ref_freq
    summary['spw']=spw
    
    # antenna
    # antenna positions
    x,y,z,antnames=antennaENU(vis)
    summary['ant']={'name': antnames, 'y': y, 'x': x, 'z': z}
    if len(summary['ant']['name']) > 45:
    
        for key_lp in ['name','x','y']:
            summary['ant'][key_lp]=summary['ant'][key_lp][0:45]

    nant=len(summary['ant']['name'])
    
    # these arrays contain...
    uv = []  # ... median bl-length in the UV plane (m)
    xy = []  # ... XY bl-length in the ENU plane (m)
    xbl = [] # ... dX coordinate of the antenna (m)
    ybl = [] # ... dY coordinate of the antenna (m)
    bl = []  # ... baseline name (ANT1-ANT2)
    
    tb.open(vis)
    for i in range(nant-1):
        for j in range(i+1, nant):
            tb1=tb.query('ANTENNA1==%i && ANTENNA2==%i%s' % (i,j,timequery))
            try:
                u,v,w=tb1.getcol('UVW')
            except:
                u,v,w=np.nan,np.nan,np.nan
            bl.append('%s-%s' % (antnames[i], antnames[j]))
            uv.append(np.median(np.hypot(u,v)))
            xy.append(np.hypot( x[i]-x[j], y[i]-y[j] ))
            xbl.append( x[i]-x[j] )
            ybl.append( y[i]-y[j] )
    tb.close()
    
    summary['bl']={'name': bl, 'uv': uv, 'xy': xy, 'dx': xbl, 'dy': ybl}

    # polarisations
    tb.open('%s/DATA_DESCRIPTION' % vis)
    pol_id=tb.getcol('POLARIZATION_ID')[spw]
    tb.close()
    
    tb.open('%s/POLARIZATION' % vis)
    pol=stokes(tb.getvarcol('CORR_TYPE')['r%i' % (1+pol_id)][:,0])
    tb.close()
    summary['pol']=pol
    
    # getting the pwv from the WVRs
    calwvr='%s/ASDM_CALWVR' % vis
    pwv = -1
    if os.path.exists(calwvr):
        tb.open(calwvr)
        water = tb.getcol('water')
        pwv = np.mean(water[np.isfinite(water)])*1000  # changed to mean - already average value PER antenna
        pwv_err = np.std((water[np.isfinite(water)])*1000)
        if np.isnan(pwv):
            pwv=-1
        tb.close()
    
    
    
    summary['pwv']=pwv
    summary['pwv_err']=pwv_err  

    # getting the weather table:
    tb.open('%s/WEATHER' % vis)
    tb1=tb.query('PRESSURE > 0 %s' % timequery)
    key ='temperature'
    tempT = tb1.getcol(key.upper())
    k=1
    if not tempT.any():
        tb1.close() # close tb1
        tdiff = timerange[1]-timerange[0]
        timequery=' && TIME >= {0} && TIME <= {1}'.format(timerange[0]-2*tdiff, timerange[1]+2*tdiff)
        tb1=tb.query('PRESSURE > 0 %s' % timequery)
    summary['weather']={}
    for key in ['time','pressure','rel_humidity','temperature','wind_speed','wind_direction','ns_wx_station_id']:
        summary['weather'][key]=tb1.getcol(key.upper())
    tb1.close()
    tb.close()
    
    # Due to new weather stations in 2015 there are some reading bad/wrong
    # pressure if found to be 500 constantly and/or wind speed is 0.
    # find stations that are ok and use these as an average only
    ID_good = np.where(summary['weather']['wind_speed']!=0)
    for key in ['time','pressure','rel_humidity','temperature','wind_speed','wind_direction','ns_wx_station_id']:
        summary['weather'][key]=summary['weather'][key][ID_good[0]]


    # get the field table
    tb.open('%s/FIELD' % vis)
    fields=tb.getcol('NAME')
    ref_dir=tb.getcol('REFERENCE_DIR')

    msmd.open(vis)
    for no_i,i in enumerate(fields):
        if 'CALIBRATE_BANDPASS#ON_SOURCE' in msmd.intentsforfield(i):
            fieldnam = i
            fieldID = no_i
    
    ra=np.zeros(1, dtype=np.float)
    de=np.zeros(1, dtype=np.float)
      

    ra[0]=np.float(ref_dir[:,0,fieldID][0])
    de[0]=np.float(ref_dir[:,0,fieldID][1])
    
    tb.close()
    msmd.close()
    summary['field'] = {'name': fields, 'ra': ra, 'de': de}
    
    # get mean azimuth and altitude of the scan (return in degrees)
    ms.open(vis)
    ss = ms.getscansummary()
    ms.close()
    
    tb.open(vis)
    tb1=tb.query('TIME > 0 %s' % timequery)
    scans = sorted( set( tb1.getcol('SCAN_NUMBER') ))
    tb1.close() # closing tb1
    tb.close() # closing tb
    
    alt = []    # contains the altitude
    az = []     # contains the azimuth
    tscan = []  # contains the scan time
    nscan = []  # contains the names
    
    me.doframe(me.observatory('ALMA'))
    for scan in scans:
        
        t0 = 1e8
        t1 = 0.
        
        for subscan,params in ss['%s' % scan].items():
            t0 = np.min( [params['BeginTime'], t0] )
            t1 = np.max( [params['EndTime'], t1] )
            fieldId = params['FieldId']
        
        t = np.mean([t0,t1])
        tscan.append(t)
        nscan.append( fields[np.int(fieldId)] )
        # convert the mean time to a datetime object
        reftime=datetime.fromtimestamp( mjd_to_unix(t), utc).strftime('%Y/%m/%d/%H:%M:%S')
        me.doframe(me.epoch('UTC', qa.quantity(reftime)))
        b=me.direction('j2000', '%.4frad' % ra[fieldId], '%.4frad' % de[fieldId])
        azel = me.measure(b, 'azel')
        alt.append(qa.convert(azel['m1'],'deg')['value'])
        az.append(qa.convert(azel['m0'],'deg')['value'])
    
    summary['scan'] = {'scan': scans, 't': tscan, 'alt': alt, 'az': az, 'name': nscan}
    
    if verbose:
        print "Finished metadata"
    
    return summary


def WVRS_Two_point_deviation_nogap(phase, time, ref_freq = 1., tau0=None,  npts=10., unwrap = False, width = None):
    '''Returns the Two-Point Deviation (\sigma) of any irregularly time-sampled distribution.
    
    This algorithm uses
    
    .. math::
       \sigma^2_y(n\tau_0, N) = \frac{1}{2 n^2\tau_0^2 (N-2n) }\,\sum_{i=0}^{N-2n-1}(x_{i+2n}-2x_{i+n}+x_i)^2
    
    the `time-series algorithm with overlapping <https://en.wikipedia.org/wiki/Allan_variance#Overlapped_variable_.CF.84_estimators>`_. The time-series is
    recursively generated from the phase-series, using :math:`y_{i+1} = \frac{x_{i+1} - x_i}{n\tau_0}` and starting with :math:`x_0 = 0`.
    
        
    .. note:: Owing to the fact that one has to interpolate over time gaps,
       this algorithm can be error-prone in the presence of non-continuous data, and should be treated with caution.
    
    .. note:: that this function returns the Two-Point Deviation
    
    :param float phase: phase-array :math:`y`
    :param float time: time-array :math:`t`
    :param float ref_freq: this parameter is obsolete, but is retained for consistency with an original Allan Variation version of the  algorithm
    :param float tau0: timescale to analyse (defaults to the median time-difference between to subsequent timestamps in the ``time`` array)
    :param float npts: The algorithm will resample the timescales, and return not more than ``npts`` points per order of magnitude.
    :param boolean unwrap: should unwrapping be performed?
    :param float width: convolution width for the phase unwrapping (see :py:func:`phaseUnwrap`)
    :return float: 2-dim array with timescales and 2-pt Deviation
    
    '''
    
    # number of samples
    n_sample = len(time)

    # normalise time, determine the time sampling (if not set), determine the
    # number of time samples
    t = np.array(time) - time[0]
    if tau0 is None:
        tau0 = np.median(np.diff(t))

    # array of the timestamps which contain a phase measurement (as multiples of the sampling time tau0)
    t_int = np.array((np.round(t/tau0)), dtype=np.int)
    
    # number of time-samples (N) and maximum scale for the sampling (nmax*tau0)
    N = np.max(t_int)+1
    nmax = np.floor((N-1)/2.)
    
    ## unwrap the phases - usally already done 
    if unwrap:
        if width is None:
            ph=phaseUnwrap2(phase)
        else:
            ph=phaseUnwrap2(phase, time=t_int, width=width)
    else:
        ph = phase

    # T is the continuous array with timestamps
    # P is the array which contains the phases. P is continuous in time, i.e.,
    # at each timestep i*tau it contains the phase value. Timestamps with no
    # measurement are NaNs
    T = np.arange(N, dtype=np.float) * tau0
    P = np.repeat( np.nan, N)
    for vt_int,vph in zip(t_int, ph):
        P[vt_int] = vph
    
    # in order to properly integrate we have to interpolate over the time gaps
    wfin = np.isfinite( P )
    wnan = np.isnan( P )
    
    # the integrated Phase IP is the cumulative sum of all phases (plus a zero at the start)
    IP = np.cumsum( np.interp( T, T[wfin], P[wfin] ) )
    IP[wnan] = np.nan
    IP = np.array([0] + [ip for ip in IP])

    # empty
    tADEV = []
    ADEV  = []

    if npts is not None:
        # this is the factor by which the binsize is going to be increased
        # in order to get the desired number of points per decade
        nfac = 10**(1./npts)

    n = 1
    while n < nmax:

        # We will use the overlapped variable tau estimator
        # https://en.wikipedia.org/wiki/Allan_variance#Overlapped_variable_.CF.84_estimators
        imax = N-2*n-1
        y = ( IP[2*n:2*n+imax] - 2*IP[n:n+imax] + IP[0:imax] )**2
        Y = np.ma.masked_array(y, np.isnan(y))
        tADEV.append(n * tau0)
        if Y.any():
            al = 0.5/n**2 * Y.mean()
            ADEV.append( al )
        else:
            ADEV.append( np.nan )
        
        if npts is None:
            n+=1
        else:
            # select the next binsize by the number of points (but it's at least n+1)
            n=np.max([n+1, np.int(np.round(n*nfac))])

    TPdev= np.array([[t, np.sqrt(a)] for t,a in zip(tADEV, ADEV)])
    
    return TPdev

def WVRS_calc2Pt(ph,time, npts=10.):
    '''LTM version of the 2pt deviation calculation
            
    :param phase: the unwrapped phases (any unit)
    :type phase: float/double array
    :param time: the time as read from the ms
    :type time: float/double array
    :param npts: The algorithm will resample the timescales, and return not more than ``npts`` points per order of magnitude.
    :type npts: float
    
    :return: 2-dim array with timescales and 2 point deviation
    
   
    '''

    # begin the code

    # deals with masked arrays which throws a warning
    # ignore these as they are given for every phase
    # array that has one or more nans
    warnings.filterwarnings("ignore")

    # zero check time

    t = np.array(time) - time[0]

    # number of samples
    n_sample = len(time)
    
    # set tau0
    tau0 = np.median(np.diff(t))

    # array of the timestamps which contain a phase measurement
    t_int = np.array((np.round(t/tau0)), dtype=np.int)
    # this is element number, not actual times

    N = np.max(t_int)+1  
    # should be number of elements with a time stamp 
    # such that N-1*diff = np.max(time)

    nmax = np.floor((N)/2.)

    # P is the array which contains the phases. P is continuous in time, i.e.,
    # at each timestep i*tau it contains the phase value. Timestamps with no
    # measurement are NaNs
    P = np.repeat( np.nan, N)
    for vt_int,vph in zip(t_int, ph):
        P[vt_int] = vph

    # create the 2pt lists
    tADEV_2 = []
    ADEV_2  = []

    if npts is not None:
        # this is the factor by which the binsize is going to be increased
        # in order to get the desired number of points per decade - log plotted
        nfac = 10**(1./npts)

    # must mask the phases as not to include them in means
    # these are 2*nmax long
    # where each are a time stamp - but will correspond to index*tau0 
    ph_m = np.ma.masked_array(P,np.isnan(P))
   
        
    n = 1 # define binsize. This is in units of tau

    while n < nmax: # wont run time past half length of time of full scan
        diffs = []
        for i in range(N-2*n+1):     # this is the calc loop (double loop)
            holder=(np.mean(ph_m[i+n:2*n+i],axis=0)-np.mean(ph_m[i:i+n],axis=0))**2
        
            diffs.append(holder)
        # these diffs have some -- values in them
        # when in mean they act as nan
        # second mask
        diffs_use = np.ma.masked_array(diffs,np.isnan(diffs))
    
        #print len(diffs)
        
        ADEV_2.append(np.mean(diffs_use))
        tADEV_2.append(n*tau0)
        if npts is None:
            n+=1
        else:
            # select the next binsize by the number of points (but it's always at least n+1)
            n=np.max([n+1, np.int(np.round(n*nfac))])


    me2pt = np.array([[ti, np.sqrt(np.array(a)/2.0 )] for ti,a in zip(tADEV_2, ADEV_2)])
    
    # set warning back on
    warnings.filterwarnings("default")
   
    return me2pt

  

def WVRS_calc2PtT(ph,time,interval):
    '''LTM version of the 2pt deviation calculation    


    :param phase: the unwrapped phases (any unit)
    :type phase: float/double array
    :param time: the time as read from the ms
    :type time: float/double array
    :param interval: time to cal 2pts over in units of tau (0.96 sec)
    :type interval: int
    
    :return: 2-dim array with a single timescale and 2 point deviation value
    
    This gives a specific 2-pt deviation for a selected time
    interval only. Not a series of 2-pt values.

    '''


    # begin the code
    # zero check time - should be done but inc here

    t = np.array(time) - time[0]

    # number of samples
    n_sample = len(time)
    
    # set tau0
    tau0 = np.median(np.diff(t))

    # array of the timestamps which contain a phase measurement
    t_int = np.array((np.round(t/tau0)), dtype=np.int)
    # this is element number, not actual times

    N = np.max(t_int)+1  
    # should be number of elements with a time stamp 
    # such that N-1*diff = np.max(time)

    nmax = np.floor((N)/2.)

    # P is the array which contains the phases. P is continuous in time, i.e.,
    # at each timestep i*tau it contains the phase value. Timestamps with no
    # measurement are NaNs
    P = np.repeat( np.nan, N)
    for vt_int,vph in zip(t_int, ph):
        P[vt_int] = vph

    # create the 2pt lists
    tADEV_2 = []
    ADEV_2  = []

    

    # must mask the phases as not to include them in means
    # these are 2*nmax long
    # where each are a time stamp - but will correspond to index*tau0 
    ph_m = np.ma.masked_array(P,np.isnan(P))
   
        
    n = interval # define binsize. This is in units of tau
    # only uses this here

    # dont need a while loop fix n value - one timescale for calc
    diffs = []
    for i in range(N-2*n+1):     # this is the calc loop (double loop)
        holder=(np.mean(ph_m[i+n:2*n+i],axis=0)-np.mean(ph_m[i:i+n],axis=0))**2
        
        diffs.append(holder)
    # these diffs have some -- values in them
    # when in mean they act as nan
    # second mask
    diffs_use = np.ma.masked_array(diffs,np.isnan(diffs))
    
    #print len(diffs)
        
    ADEV_2.append(np.mean(diffs_use))
    tADEV_2.append(n*tau0)
    # this is returns, only single time, single value for the 2pt

    me2pt = np.array([[ti, np.sqrt(np.array(a)/2.0 )] for ti,a in zip(tADEV_2, ADEV_2)])
    
    

    return me2pt


def WVRS_two_point_deviation_func(dataset, prefix=None, timerange=None, meta=None, spw=None, unwrap=True, verbose=True, algorithm='calc2pt', wvr_outlier=False, single=True, flag_outlier='',lim=90, over=20, std_val=3.5, auto=False, rapid=False,columns=False, fitsonly=False, Flags=True):
    '''Analyse the data products and generates fits tables with the Two-Point Deviation for each dataset.
       For PI science data single should be true, this just unwraps wrapped phases, i.e. those exceeting +180 or -180 degrees. If this is False, a wrap checker will be used, this uses the inputs 

       lim, over, std_val, auto, rapid 

       The defaults work well but these can be adjusted - see help for wrapCheckTimeRMS
    
       The wrapping checker is to avoid having to flag phase delay jumps and is left in this code as it formed part of a group of funciton used in phase metrics tests, with custom ALMA test data. HOWEVER, in PI data flagging of the data is ALWAYS RECOMMENDED rather than using the wrapping checker. Flags in the ms are correctly used and extra time ranges to flag can ALSO be used.


    :param str dataset: input filename
    :param str prefix: prefix for the output filenames - if none, results not saved or returned
    :param float timerange: timerange is a list [t_min, t_max], where t_min/t_max are
        the time since the start of the observations
    :param dict meta: dictionary with metadata (see the metadata function)
    :param str spw: spectral window to analyse (default: find last chan-averaged spw that is wide enough) defualt = None
    :param boolean unwrap: unwrap the phases or not 
    :param boolean verbose: verbose?
    :param str algorithm: algorithm to calculate the Two Point Deviation
    :param boolean wvr_outlier: checks and does some auto flagging based on outliers with STD calculation
    :param boolean single: option to do single unwrap only pi-pi limit with no wrapcheck
    :param string flag_outlier: string with the times to flag out e.g 12~30,40~50
    :param int lim:
    :param int over: 
    :param int std_val: 
    :param boolean auto: calculates the lim based on the STD of the data
    :param boolean rapid: uses RMS check with 30s average not 60 if data varies rapidly
    :param boolean columns:
    
    
    :returns: If no file prefix is given, nothing is returned
    :rtype: dict
    
    '''
    
    # if we have a calibration table, we have to extract the column "CPARAM",
    # otherwise it'll be the "DATA" and "CORRECTED_DATA" columns

    data_type = dataset.split('.')[-1]

  

    calc_pwv = False
    if data_type == 'pwv':
        #data_type = 'ms'
        calc_pwv = True
        spw_column_name = 'DATA_DESC_ID'
        columns = ['CPARAM']
    elif data_type == 'wvr':
        columns = ['CPARAM']
        spw_column_name = 'SPECTRAL_WINDOW_ID'
        spw_wvr=True
        print 'SETTING SPW_WVR to TRUE'
#        bl_ant = 'ANT'
    else:
        if 'unity' in dataset:
            columns = ['CORRECTED_DATA']
        else:
            if columns:
                columns=columns  # input if required for manual read of only data 
                data_type = 'ms'
            else:
                columns = ['DATA', 'CORRECTED_DATA']  # note may be able to invert if using wvr corrected as proxy for outliers if data has del jumps
        spw_column_name = 'DATA_DESC_ID'
        spw_wvr=False
        
        ##print 'SETTING SPW_WVR to FALSE'
#        bl_ant = 'BL'

    # set metadata
    if meta is None:
        M = WVRS_metadata(dataset)
    else:
        M = meta
    
    # this shouldnt occur as it is limited in the Meta data file 
    if len(M['ant']['name']) > 45:
     	nant = 45  # max out fits of 1000 columns so far - cant tell 'which' antennas we loose here
    else:					
        nant=len(M['ant']['name'])
    ##nant = 45  ## must hard fix for data with more than 45 ants - fits files max out
    # boolean mask for each pol now in function
    polavg = getPolmask(M)

    polstr = Join(np.array(M['pol'])[np.array(polavg) == False],',')
    
    # call the time out of the loop
    # for wvr or ms data as its same for all
    if spw is None:
        spwnum = M['spw']  ## this now is from the correct method
    else:   ## if put in manually
        spwnum = spw
        M['spw'] = spw 
    #if spw:
    #    spwnum=spw
    #    M['spw']=spw
    #else:
    #    spwnum = getLastSpwID_ms()  ## 8 is standard but HLtau B7 is 20 data needs changing,  M['spw'] usually - coded in input now 

   

    if verbose:
        if fitsonly:
            print " * Populating FITS files with phases for WVR scaling ONLY "
        else:
            print " * Calculating Two-Point-Deviations, alg = {0}".format(algorithm)
        print "   - filename:     %s" % dataset
        if calc_pwv:
            print "   - type:         pwv"
        else:
            print "   - type:         %s" % data_type
        print "   - antennas:     %i  " % nant
        print "   - spw:          %i" % (M['spw'])
        print "   - frequency:    %.2f GHz" % (M['ref_freq']*1e-9)
        print "   - Band:         %i" % M['band']
        print "   - pwv:          %.2f mm" % M['pwv']
        print "   - polarisation: %s" % polstr

    # setup for the results
    wvr={}
    results = {}
    data    = {}
    tbdata  = {}
    tbres   = {}
    tbmeta  = []
    for column in columns:
        results[column]={}
        data[column]={}
        tbdata[column]=[]
        tbres[column]=[]
#    results[bl_ant]={}


    # choosing the algorithm
    # only the main 3 now
    # all equivant but Hills 3 much faster
    # but doesnt strictly deal with gaps
    if  algorithm == 'calc2pt_nogap':
        ALLG=WVRS_Two_point_deviation_nogap
    elif algorithm == 'calc2pt':
        ALLG=WVRS_calc2Pt

       
       
    
    pass_pols = M['pol']

    # already can tell here if it is a wvr or ms/unity file - spw_wvr TRUE/FALSE
   
    if not calc_pwv:
        #print spw_wvr, spwnum
        T = getTime_2pt(dataset,ant1=0,ant2=1,WVR=spw_wvr,spwnum=spwnum)   
    
    # string for the query for the phases in loop
    #query='{0} == {1}{2}'.format(spw_column_name, M['spw'],M['timequery'])
    query='{0} == {1}{2}'.format(spw_column_name, spwnum,M['timequery'])

    # the outer loop - check if we are comparing baseline data or antenna data eg, wvr
    iloop = np.arange(nant-1) if data_type == 'ms' else np.arange(nant)
  
    

    # open dataset here
    tb.open(dataset)
    
    for i in iloop:
        
        # this loops over all antennas



        # option for double loop if baseline based, not for antenna based
        jloop = np.arange(i+1, nant) if data_type == 'ms' else [-1]
        # if verbose:
            # print "     %s" % M['ant']['name'][i]
        for j in jloop:
            
            # required for the filling dict at end
            data_id = '{0}-{1}'.format(M['ant']['name'][i], M['ant']['name'][j]) if data_type == 'ms' else M['ant']['name'][i]
        
            for column in columns: #(data, corrected, cparam)
                ##print '##################'
                ##print ' doing ', column
                ##print ' #########'
            
                
    
                # if pwv run seperate function
                if calc_pwv:
                    tmpT, tmpP=pwvFromMS(dataset, antenna=[i])
                    T = tmpT[i]
                    P = tmpP[i]
                    T -= T[0] # normalising the time
                else:
                    # create a subtable with the query parameters and pass to getPhase_2pt

                    subtb=tb.query('ANTENNA1=={0} && ANTENNA2=={1} && {2}'.format(i, j, query), sortlist="TIME", columns="TIME, FLAG, {0}".format(column))
                    
                    ph =  getPhase_2pt(subtb,column,polavg,spw_wvr,pass_pols, Flags=Flags)
                    
                    # phase is now both XX and YY passed

                    # also add in the flag_outlier variable
                    if flag_outlier:
                        ph = flagger_remove(ph,T,flag_out=flag_outlier)  # passed phase and times
                        flag_out_bool = True

                    # check now if outlier removal is requested and only do for CORRECTED_DATA
                    # default is false - was testing code - leave in the option
                    if wvr_outlier == True and column == 'CORRECTED_DATA':
                        print ' Option selected to remove the outliers in the WVR corrected data'
                        ph = outlier_remove(ph,factor=3.0)

                    # unwrap if requested - default
                    if unwrap:
                        # if unwrappping one, then assume other needs unwrap too
                        if len(ph) == 2 or len(ph) == 4:
                            ## check for nans as this will not work otherwise
                            if len(ph[0][0][np.isfinite(ph[0][0])]) > 1:

                                if (np.max(ph[0][0][np.isfinite(ph[0][0])]) > 3.05 or np.min(ph[0][0][np.isfinite(ph[0][0])]) < -3.05) or (np.max(ph[1][0][np.isfinite(ph[1][0])]) > 3.05 or np.min(ph[1][0][np.isfinite(ph[1][0])]) < -3.05):
                                    for ii in range(len(ph)):
                                        ph[ii][0] = phaseUnwrap2(ph[ii][0],factor=1.2)  # deals with nan gaps - only unwraps if required
                                    ph_un = np.mean(ph,axis=0)[0]  # for data or corrected need to specify index to get array
                             
                                    if not single:
                                    # does the wrap check - default is single only needed for 
                                    # jumpy - delayed test data
                                    # also flagging is recommended rather than trying to catch the bad
                                    # times where incorrect un-wrapping is done
                                
                                        P = wrapCheckTimeRMS(ph_un,T,lim=lim, over=over, std_val=std_val,auto=auto,rapid=rapid) # can opt out by saying single = True, this now also deals with nan values
                                       
                                    else:
                                        P = ph_un
                                else:
                                    P = np.mean(ph,axis=0)[0] # if it has not gone through the unwrap code - just average XX YY 
                                    
                            # passes the nan values as doesnt conform to the unwrap checks
                            else:
                                P = np.mean(ph,axis=0)[0]

                        elif len(ph) > 10:
                            ##check for nans as this will not work otherwise
                            if len(ph[np.isfinite(ph)]) > 1:
                                if (np.max(ph) > 3.05 or np.min(ph) < -3.05):
                                    ph_un = phaseUnwrap2(ph)  # for wvr one array already
                               
                                    if not single:
                                        P = wrapCheckTimeRMS(ph_un,T,lim=lim, over=over, std_val=std_val,auto=auto,rapid=rapid)
                                    #P = wrapCheckTime(ph_un,T) # usually no issues with WVR or PWV streams
                                    else:
                                        P = ph_un
                                else:
                                    P = ph  # if we ask for unwrap but not required
                            else:
                                P = ph  # if it is just nan values 
                        else:
                            print ' Fetching and unwrapping phases has failed'
                            print ' for this baseline - the input data is likely all bad'
                            print ' '+M['ant']['name'][i]+'-'+M['ant']['name'][j]


                    else:   #  no wrap requested
                        print ' You dont want to unwrap'
                        if len(ph) == 2 or len(ph) == 4:
                            P = np.mean(ph,axis=0)[0]
                        else:
                            P = ph
                                         
                # write the phase into the data array - time and correct unwrapped phases

                
                data[column][data_id] = np.array([[t,p] for t,p in zip(T,P)])
                
                dT=np.median(np.diff(T))
                T=np.round(T/dT) * dT
                # write the 2-point deviations in a results array - pointer to function
                if not fitsonly:
                    # this does the TPD calculations which can be slow, for fits only this runs the phase population
                    results[column][data_id] = ALLG(P, T)
                    # will ignore if fitsonly is set 
    tb.close() 
    
    if prefix is not None:

           
        ## generate the metadata extension
        if verbose: print " * Writing FITS table"
        
                 

        prihdr = fits.Header()
        prihdr['PWV'] = M['pwv']
        prihdr['PWVERR'] = M['pwv_err']
        prihdr['RESTFREQ'] = (M['ref_freq'], 'Hz')
        prihdr['BAND'] = M['band']
        prihdr['SPW'] = M['spw']
        prihdr['ALLAN'] = (data_type, 'MS: baseline; WVR: antenna')
        prihdr['DATE-OBS'] = (M['date-obs'], 'UTC')
        prihdr['POL'] = polstr
        #prihdr['FLUXDENS'] = np.mean( M['scan']['flux_density'] ) # no longer looked into - no needed here
        if not fitsonly:
            prihdr['SOURCE'] = M['scan']['name'][0]
            prihdr['ALT'] = np.mean( M['scan']['alt'] )
            prihdr['AZ'] = np.mean( M['scan']['az'] )
        prihdr['N_EXT'] = (3+2*len(columns)+len(columns)-1,'number of extensions')
        
        # an overview of the extensions:
        #  1 - antenna parameters (names, positions)
        #  2 - baseline parameters (names, baseline length)
        #  3 - weather parameters
        #  4 - phase
        #  5 - Two Point deviation
        # (6 - phase)
        # (7 - Two Point deviation)
        # (8 - Phase RMS)
        
        prihdu = fits.PrimaryHDU(header=prihdr)
        tb0=[prihdu]
        #print " 1-stop"
        # extension 1: antenna
        tbtmp=[]
        for key,fmt in zip(['name','x','y'],['20A','E','E']):
            tbtmp.append( fits.Column(name=key, format=fmt, array=M['ant'][key]) )
        #tb0+=[fits.new_table(fits.ColDefs(tbtmp))] #depriciated edit Aug2017
        tb0+=[fits.BinTableHDU.from_columns(fits.ColDefs(tbtmp))]
        tb0[-1].header['EXTNAME']='ANT'
        prihdr['EXT_1']='ANT'
        #print " 2-stop"
        # extension 2: baseline
        tbtmp=[]
        for key,fmt in zip(['name','uv','xy'],['20A','E','E']):
            tbtmp.append( fits.Column(name=key, format=fmt, array=M['bl'][key]) )
        #tb0+=[fits.new_table(fits.ColDefs(tbtmp))] #depriciated edit Aug2017
        tb0+=[fits.BinTableHDU.from_columns(fits.ColDefs(tbtmp))]
        tb0[-1].header['EXTNAME']='BL'
        prihdr['EXT_2']='BL'

        #print " 3-stop"

        # extension 3: weather
        tbtmp=[]
        if not fitsonly:
            for key,fmt in zip(['wind_speed', 'temperature', 'wind_direction', 'rel_humidity', 'pressure',  'time', 'ns_wx_station_id'], ['E','E','E','E','E','E','I']):
                tbtmp.append( fits.Column(name=key, format=fmt, array=M['weather'][key]) )
            #tb0+=[fits.new_table(fits.ColDefs(tbtmp))] #depriciated edit Aug2017
            tb0+=[fits.BinTableHDU.from_columns(fits.ColDefs(tbtmp))]
            tb0[-1].header['EXTNAME']='WEATHER'
            prihdr['EXT_3']='WEATHER'
        
        iext=4
        # extension 4 to 7
        #print columns

        for column in columns:
            #print " "+str(iext)+"-stop"
            #print " "+str(iext+1)+"-stop"

            if column == 'DATA':
                app = ''
            else:
                app = '_CORR'
            
            ttpd=[]
            tbphase=[]
            
            first=True
            for key in data[column]:
                #print key
                if first:
                    if not fitsonly:
                        ttpd.append( fits.Column(name='tscale', format='D', array=results[column][key][:,0]) )
                    tbphase.append( fits.Column(name='time', format='D', array=data[column][key][:,0]) )
                    first=False
                if not fitsonly:
                    ttpd.append( fits.Column(name=key, format='D', array=results[column][key][:,1]) )
                tbphase.append( fits.Column(name=key, format='D', array=data[column][key][:,1]) )
            #print 'out of loop - key is BL'

            #tb0.append(fits.new_table(fits.ColDefs(tbphase)))# depriciated edit Aug2017
            tb0.append(fits.BinTableHDU.from_columns(fits.ColDefs(tbphase))) 
            tb0[-1].header['EXTNAME']='PHASE%s' % app
            prihdr['EXT_%i' % (iext)]='PHASE%s' % app
       
     
            if not fitsonly:
                #tb0.append(fits.new_table(fits.ColDefs(ttpd)))# depriciated edit Aug2017
                tb0.append(fits.BinTableHDU.from_columns(fits.ColDefs(ttpd)))
                tb0[-1].header['EXTNAME']='ADEV%s' % app
                prihdr['EXT_%i' % (iext+1)]='ADEV%s' % app
            
            iext+=2

        # Extension 8: Phase RMS (stored in radians)
       
        if len(columns) > 1:
            tbtmp=[]
            phaserms = getSTDall(data)  ## nov 2016 - this is fine as is linear correcting - single offset for the data
            for key,fmt in zip(['name','phase_rms','corr_rms'],['20A','E','E']):
                tbtmp.append( fits.Column(name=key, format=fmt, array=phaserms[key]) )
            #tb0+=[fits.new_table(fits.ColDefs(tbtmp))]
            tb0.append(fits.BinTableHDU.from_columns(fits.ColDefs(tbtmp)))
            tb0[-1].header['EXTNAME']='PHASE_RMS'
            prihdr['EXT_8']='PHASE_RMS'

        
        thdulist = fits.HDUList(tb0)
        fitsfile = '%s.fits' % (prefix)
        if os.path.exists(fitsfile):
            os.system('rm %s' % fitsfile)
        thdulist.writeto(fitsfile, clobber=True)
        
    else:
        return results




def phaseUnwrap2(phase, time=None, factor=1.0): 
	''' Unwraps the phases 
	
	:param phase: phase in radians
	:type phase: float array
	:param time: time
	:type time: float array
	:returns: unwrapped phase-array
	:rtype: float array
	
	If no time array is given, the phase is simply unwrapped by
	identifying phase jumps by more than 180deg. That can be quite
	tricky for noisy data - historic but retained - not used.
	
        Modified version where 
        1) factor can be used to specify wraps (multiples of pi)
        2) nans are dealt with correctly

	Note is slower, 65 ms vs 200 us
 
        
	'''
	
	
	phase2 = phase.copy()
	
	if len(phase2[np.isfinite(phase2)]) > 2:
		if np.isnan(phase2[0]):
			phase2[0] =[xp for xp in phase2 if not np.isnan(xp)][0]
		
		phase2 = phase2[np.isfinite(phase2)]
		diff   = np.diff(phase2)   ## this is a shorted array - not smooth in time
		pup    = np.cumsum(diff < -factor*np.pi)
		pdown  = np.cumsum(diff > factor*np.pi)


		ph = np.zeros_like(phase)
		ph[0] = phase[0]  
	

		# now need to make pup and pdown related to original phase
		# but they are shorter arrays

		lp = 1
		cr = 0
		correct = pup-pdown
		while lp < len(phase):
		# now correct on the fly with the 
		# first element of pup-pdown
		# then remove the element
		# if the main array is not a nan and correction was applied
		# have to do this way as the output is a phase
		# stream to be averaged with another 

			if np.isfinite(phase[lp]): 
				ph[lp] = correct[cr] * 2*np.pi + phase[lp]
				cr=cr+1
			else:
			# i.e. no correction as phase is a nan - so just fill ph with a nan
				ph[lp] = phase[lp]

			lp=lp+1
	else:
		ph = phase2
		
	return ph
	

def WVRSminScaling(time_in,ph_in,un_in,wvr_in,tpd,lim_up=False):
    ''' The function reads in the phase stream and 
        acts to minimise a scaling factor according to
        the given timescale of the tpd OR rms values

        assumes that there is a smooth minimum 
        such that coarse looping is used to narrow down the value
       
        :param time_in: the time stream
        :type time_in: float array
        :param ph_in: the phase stream
        :type ph_in: float array
        :param un_in: the unity file (correction) stream
        :type un_in: float array
        :param wvr_in: the wvr corrected stream 
        :type wvr_in: float array
        :param tpd: the two-point-deviation timescale (or RMS call)
        :type tpd: float or string
	:param lim_up: Range of scalings to use. False for standard range, True runs to 0-5
	:type lim_up: boolean
	


	:returns: the scale value
	:rtype: float
	'''

    

    # int time average 
    d_time = np.diff(time_in)
    
    val_use = np.round(tpd/np.median(d_time))
    val_use = np.int(val_use)
    
    
    # loop was from 0.05 to 2.5 before - in 0.01 (245 in the loop)
    # new limits 0.2 to 2.4 in 0.2 then narrow
    if lim_up:
	    scales_cor = [0.2,0.4,0.6,0.8,1.0,1.2,1.4,1.6,1.8,2.0,2.2,2.4,2.6,2.8,3.0,3.2,3.4,3.6,3.8,4.0,4.2,4.4,4.6,4.8,5.0]
    else:
	    scales_cor = [0.2,0.4,0.6,0.8,1.0,1.2,1.4,1.6,1.8,2.0,2.2,2.4]

    scales_h=[]
    for sc_c in scales_cor:
        scales_h.append(WVRS_calc2PtT(ph_in+un_in*sc_c,time_in,val_use)[:,1][0])
	   

    # find the minimum and then finer resolution
    sc_ID = [index for index,value in enumerate(scales_h) if value == np.min(scales_h)][0]
    # scaling value to push forward - and narrow the range
    sf = scales_cor[sc_ID]
	    
    scales_cor2 = [sf-.18,sf-.14,sf-.10,sf-.06,sf-.02,sf+.02,sf+.06,sf+.1,sf+.14,sf+.18]

    scales_h=[]
    for sc_c in scales_cor2:
        scales_h.append(WVRS_calc2PtT(ph_in+un_in*sc_c,time_in,val_use)[:,1][0])
	   
    # find the minimum and then final finest resolution
    sc_ID = [index for index,value in enumerate(scales_h) if value == np.min(scales_h)][0]
    # scaling value to push forward - even narrower range
    sf = scales_cor2[sc_ID]
	    
    scales_cor3 = [sf-.03,sf-.02,sf-.01,sf,sf+.01,sf+.02,sf+.03]
    scales_h=[]
    for sc_c in scales_cor3:
        scales_h.append(WVRS_calc2PtT(ph_in+un_in*sc_c,time_in,val_use)[:,1][0])
	    
    # find the minimum of this last loop
    sc_ID = [index for index,value in enumerate(scales_h) if value == np.min(scales_h)][0]
    # scaling value to return
    sf = scales_cor3[sc_ID]
    
    return sf


