## this is the set of plotting code
## Developed at Allegro, ALMA Arc node Leiden 


from wvrScaleImportFuncs import *
from wvrScaleAnalysis import *

def WVRS_plotTPDS(results, meta=None, data=None, nplot=4, plot='BOTH', TeX=False, figdir=None, overwrite=True, xlim=None, ylim=None, automargin=True, eps=False, figsize=[5.6, 7.5], badant=''):
    ''' Dedicated function to plot the Two-Point-Deviations, Phase and the antenna/baseline configuration.
    
    :param results: dictionary with results returned from - two_point_deviation_func - or a filename of the binary fitstable (of correct type as two_point_deviation_func
    :type results: dict or str
    :param dict meta: dictionary with metadata (as returned from ``metadata(...)``)
    :param dict data: if ``results`` is the dictionary with the Two Ppint Deviations, the dictionary with the original phase values can be provided as this variable. If ``results`` is a binary fitstable, ``data`` is automatically extracted from the file
    :param int nplot: number of antennas/baselines per page
    :param str plot: mainly important for the measurement set. If ``plot='BOTH'``, the plots show both the original, the corrected, and the improvement. Other options are ``'DATA'`` or ``'CORRECTED_DATA'`` for a measurement set, or ``'CPARAM'`` for a WVR table.
    :param boolean TeX: use TeX fonts
    :param str figdir: directory where the plots have to be stored (if at all)
    :param boolean overwrite: overwrite existing plots
    :param float xlim: x limits for the Two-Point-Deviation plot
    :param float ylim: y limits for the Two-Point-Deviation plot
    :param boolean automargin: Automatically find x- and y-limits for the dataset (including some margin)
    :param boolean eps: generate also eps-versions of the plot
    :param float figsize: list with the image size (in inches)
    :param string badant: choose antennas to ignore in plots
    '''
    
    # set up the parameters for the plots
    params = {'backend': 'eps',
        'axes.labelsize': 8,
        'font.size': 6,
        'legend.fontsize': 8,
        'xtick.labelsize': 6,
        'ytick.labelsize': 6,
        'font.family': 'sans',
        }
    # color parameter maps
    cmaps=[cm.Greys, cm.Blues, cm.Greens, cm.Reds, cm.Purples, cm.Oranges]
    
    pl.rcParams.update(params)
    if TeX:
        pl.rcParams.update({'text.usetex': True, 'font.family': 'serif', 'font.serif': 'Helvetica'})
    
    # convert figsize into a numpy array
    figsize=np.array(figsize)
    
    plot2=None
    omag=1.
    if plot == 'BOTH':
        plot='DATA'
        plot2='CORRECTED_DATA'
        omag=1.
    
    if figdir is not None:
        if os.path.exists(figdir) and overwrite:
            pa = figdir.split('/')
            if len(pa) > 1:
                os.chdir(Join(pa[:-1],'/'))
                os.system('rm -rf %s' % pa[-1])
                os.chdir(Join(np.repeat('..',len(pa)-1),'/'))
            else:
                os.system('rm -rf %s' % figdir)
            os.system('mkdir -p %s' % figdir)
            if eps:
                os.system('mkdir -p %s/eps' % figdir)
        if not os.path.exists(figdir):
            os.system('mkdir -p %s' % figdir)
            if eps:
                os.system('mkdir -p %s/eps' % figdir)

    pwv='pwv: nan'
    restfreq=-1
    intent=''
    spw=''
    band=0
    pwvval=-1.
    flagants = badant.split(',')
    
    
    if type(results) is str:
        try:
            uid,data_type,suffix=results.split('.')
        except ValueError:
            try:
                uid,name_addition,data_type,suffix=results.split('.')
                uid=uid+'.'+name_addition
            except ValueError:
                try:
                    uid,name_addition1,name_addition2,data_type,suffix=results.split('.')
                    uid=uid+'.'+name_addition1+'.'+name_addition2
                except ValueError:
                    uid,name_addition1,name_addition2,name_addition3,data_type,suffix=results.split('.')
                    uid=uid+'.'+name_addition1+'.'+name_addition2+'.'+name_addition3

#        toplabel=results.split('.')[0]
        d=fits.open(results)
        pwvval=d[0].header['PWV']
        if pwvval > 0:
            pwv='pwv: %.2f mm' % pwvval
        
        restfreq='f: %.1f GHz' % (d[0].header['RESTFREQ']*1e-9)
        restfreq_val = d[0].header['RESTFREQ']
        bl_ant=d[0].header['ALLAN'].strip().upper()
        spw='spw: %i' % d[0].header['SPW']
        band=d[0].header['BAND']
        
        toplabel='Band%i, %s' % (band,pwv) 
        headlabel=Join([uid,restfreq],' - ')
#        %s   --- freq=%s    ---   pwv=%s
        # get the conversion:
        
        if bl_ant == 'MS':
            conv = {'ADEV_CORR': ['results','CORRECTED_DATA'],
              'ADEV': ['results','DATA'],
              'PHASE_CORR': ['data','CORRECTED_DATA'],
              'PHASE': ['data','DATA']}
        else:
            conv = {'ADEV_CORR': ['results', 'CPARAM'],
              'PHASE_CORR': ['data', 'CPARAM']}
        
        # metadata
        M={}
        
        # query all extensions
        extensions=['']
        for i in range(1,len(d)):
            extensions.append( d[i].header['EXTNAME'] )
        
        results={}
        data={}
        
        # antenna information?
        plot_antenna = False
        plot_weather = False
        plot_corrected = False
        plot_original = False
        
        for i,val in enumerate(extensions):
            
            D = d[i].data
            
            if val == 'ANT':
                plot_antenna = True
                M['ant'] = {}
                antenna_data = D.copy()
                M['ant']['name'] = antenna_data['name']
                M['ant']['x'] = antenna_data['x']
                M['ant']['y'] = antenna_data['y']
            elif val == 'WEATHER':
                plot_weather = True
                weather_data = D.copy()
        
            elif val == 'BL':
                M['bl'] = {}
                bl_data = D.copy()
                M['bl']['name'] = bl_data['name']
                M['bl']['xy'] = bl_data['xy']
                M['bl']['uv'] = bl_data['uv']
            elif val == 'PHASE_RMS':
                M['rms'] = {}
                phase_data=D.copy()
                M['rms']['name']=phase_data['name']
                M['rms']['phase']=phase_data['phase_rms']
                M['rms']['corr']=phase_data['corr_rms']
            elif val in conv.keys():
                if '_CORR' in val:
                    plot_corrected = True
                else:
                    plot_original = True
                todo=conv[val]
                if todo[0] == 'results':
                    results[todo[1]]={}
                    upd = results[todo[1]]
                else:
                    data[todo[1]]={}
                    upd = data[todo[1]]
            
                xunit = D.names[0]
                for col in D.names[1:]:
                    upd[col]=np.array([D[xunit],D[col]]).transpose()
        
        meta = M

    # generate some other things from the meta data
    if 'ant' in meta.keys():
        plot_antenna=True
        antpos={}
        for iname, ix, iy in zip(M['ant']['name'], M['ant']['x'], M['ant']['y']):
            antpos[iname]=[ix,iy]

   


    if data is not None:
        plot_phase = True
    else:
        plot_phase = False

    if bl_ant == 'MS':
        if plot_corrected:
            plot='CORRECTED_DATA'
        else:
            plot='DATA'
            
        if plot_corrected & plot_original:
            plot='BOTH'
        unit='m'
        
        
        # use uv 
        blsort = sorted(zip(M['bl']['uv'], M['bl']['name']))  # xy is ground not projected
    else:
        plot='CPARAM'
        unit=''
        
        blsort = zip(range(len(M['ant']['name'])), M['ant']['name'])

#    pl.close(1)

        
    pressure = np.median(weather_data['pressure'])
    temperature = np.median(weather_data['temperature'])-273.15
    winddir = np.median(weather_data['wind_direction'])
    windspeed = np.median(weather_data['wind_speed'])

    # plotting the weather (switched off by default)
    if False:
        pl.close(2)
        fig=pl.figure(2, figsize=(3.44, 3.44))
        ax=fig.add_subplot(111, projection='polar')
        for i,station in enumerate(set(weather_data['ns_wx_station_id'])):
            w=weather_data['ns_wx_station_id'] == station
            
            for j,val in enumerate(zip(weather_data['wind_direction'][w], weather_data['wind_speed'][w])):
                xy=(np.radians(val[0]),val[1])
                col=cmaps[i](1-j/(np.sum(w, dtype=np.float)))
                ax.plot(np.pi/2.-xy[0], xy[1], 'o', ms=2*xy[1], color=col)
        ax.plot(0,0,'o', color='k',zorder=2)
        if figdir is not None:
            headlabel2='p: %i mbar | T: %.1f C' % (pressure, temperature)
            headlabel='%s | %s' % (headlabel, headlabel2)
            fig.savefig('tmp_wind.png', dpi=300.)
            os.system('convert  -font FreeSerif-Medium -size x50 -background DarkRed -fill white -pointsize 15 -density 200 -gravity SouthWest label:" %s" label1.png' % (headlabel2))
            os.system('convert  -font FreeSerif-Bold -size x100 -background IndianRed -fill white -pointsize 20 -density 200 -gravity SouthWest label:" %s" label2.png' % (toplabel))
            os.system('convert -background DarkRed label1.png tmp_wind.png -append tmp.png')
            os.system('convert -background IndianRed label2.png tmp.png -append {0}/weather.png'.format(figdir))
            os.system('rm label*png tmp*png')

    if xlim is None:
        xlim=[ np.nan, np.nan ]
    if ylim is None:
        ylim=[ np.nan, np.nan ]
    ylimB=None

    if automargin:
        
        dx=[]
        dy=[]
        
        drx=[]
        dry=[]

        if plot == 'BOTH':
            dtypes = ['DATA','CORRECTED_DATA']
            doratio = True
        else:
            dtypes = [plot]
            doratio = False

        for bl_length,baseline in blsort:
            
            #print baseline
            ## do checks here for bad baselines and ants
            bl_combo1 = baseline.split('-')[0]
            try:
                bl_combo2 = baseline.split('-')[1]
            except IndexError:
                bl_combo2 = 'ANT'
            

            if bl_combo1 not in flagants and bl_combo2 not in flagants:
                #print 'doing this??'
                for pp in dtypes:
                    d = results[pp][baseline]
                    dx+=[i for i in d[:,0]]
                    dy+=[i for i in d[:,1]]
            
                if doratio:
                    d = results['DATA'][baseline]/results['CORRECTED_DATA'][baseline]
                    dry+=[i for i in d[:,1]]
        dy=np.array(dy)
        dy = toPathlength(restfreq_val,dy,verbose=False)
        dx=np.array(dx)
        w=np.isfinite(dy)
        xlim = [ np.min(dx), np.max(dx) ]
        ylim = [ np.nanmin(dy), np.nanmax(dy) ]
#        ylim = [ np.max( [ np.min(dy[w]), np.mean(dy[w]) - np.std(dy[w])*5] ), np.min([ np.max(dy[w]), np.mean(dy[w]) + np.std(dy[w])*5 ]) ]
        
        dx = ( np.log10(xlim[1]) - np.log10(xlim[0]) ) * 0.05
        xlim = np.array(xlim)*10**(dx*np.array([-1,1]))
        dy = ( np.log10(ylim[1]) - np.log10(ylim[0]) ) * 0.05
        ylim = np.array(ylim)*10**(dy*np.array([-1,1]))

        if doratio:
            dry=np.array(dry)
            w=np.isfinite(dry)
            ylimB = [0, np.min([ np.nanmax(dry), np.mean(dry[w]) + np.std(dry[w])*5])]

    iplot=0 # counter for the curves per page
    Iplot=0 # counter for the total number of curves
    Nplot=len(blsort) # number of plots
    inplot=1
    first=True
    
    if plot != 'DIFF':
        ylim[0] = ylim[0]/10**((nplot-1)/omag)
    
    # loop over the baseline sorted by length
    """
     Option 1 : Antenna - based
            Sort by Antenna ID
        blength = Antenna ID
        baseline = Antenna Name
    
     Option 2 : WVR baseline
        Sort by baseline and plot WVR
        blenght = baseline length
        baseline = baseline name (Ant1-Ant2)

    Option 3 : Phase baseline
        Sort by baseline and plot phase corrected and uncorrected
        blenght = baseline length
        baseline = baseline name (Ant1-Ant2)
    
    """
    print " ** Start plotting Two-Point-Deviations"
    for blength, baseline in blsort:

        ## do checks here for bad baselines and ants
        bl_combo1 = baseline.split('-')[0]
        try:
            bl_combo2 = baseline.split('-')[1]
        except IndexError:
            bl_combo2 = 'ANT'

        if bl_combo1 not in flagants and bl_combo2 not in flagants:
        # print iplot, Iplot
        # print baseline,
            if iplot >= nplot or first:
            
            # define a new figure if you are the very first
                if first:
                    if pl.fignum_exists(1):
                        fig = pl.figure(1, figsize=figsize)
                        fig_wh = np.array([ fig.get_figwidth(), fig.get_figheight() ])
                        if not np.all(figsize-fig_wh == [0,0]):
                            pl.close(1)
            
                iplot=0 # reset the counter if you have done more than nplot baselines
                fig=pl.figure(1, figsize=figsize)
                fig.clf()
                # Setting up the head lines
                if data_type == 'vis':
                    hcol=cm.Blues
                elif data_type == 'wvr':
                    hcol=cm.Reds
                else:
                    hcol=cm.Oranges
                ax0=fig.add_axes([0., 0.93, 1., 0.07], axisbg=hcol(0.5))
                if TeX:
                    ax0.text(0.03, 0.1, r'\textbf{%s}' % toplabel.replace('_','\_'), fontsize=18, color='w', transform=ax0.transAxes)
                    ax0.text(0.97, 0.1, r'\textbf{%s}' % data_type, fontsize=25, color='w', transform=ax0.transAxes, ha='right', va='bottom')
                else:
                    ax0.text(0.03, 0.1, toplabel, fontsize=18, color='w', transform=ax0.transAxes)
                    ax0.text(0.97, 0.1, data_type, fontsize=25, color='w', transform=ax0.transAxes, ha='right', va='bottom')                
                ax0.xaxis.set_ticks([])
                ax0.yaxis.set_ticks([])
#            ax0.axis('off')
                ax0=fig.add_axes([0., 0.89, 1., 0.04], axisbg=hcol(0.8))
                if TeX:
                    ax0.text(0.03, 0.1, headlabel.replace('_','\_'), fontsize=8, color='w', transform=ax0.transAxes)
                else:
                    ax0.text(0.03, 0.1, headlabel, fontsize=8, color='w', transform=ax0.transAxes)
                ax0.xaxis.set_ticks([])
                ax0.yaxis.set_ticks([])
#            ax0.axis('off')
            
            # setting up the first image
                if plot == 'BOTH':
                    gs1=GS(2,1, height_ratios=[2.5,1.])
                    ax1a=fig.add_subplot(gs1[0])
                    ax1b=fig.add_subplot(gs1[1])
                else:
                    gs1=GS(1,1)
                    ax1a=fig.add_subplot(gs1[0])
                gs1.update( left=0.10, right=0.47, hspace=0.02, bottom=0.05, top=0.8)

                blmin=blength
            
            # setting up the second image
                if plot_phase:
                
#                nx2,ny2=np.int(np.floor(np.sqrt(nplot))), np.int(np.ceil(np.sqrt(nplot)))
                    nx2=1
                    ny2=nplot
                    gs2=GS(ny2, nx2)
                    gs2.update(left=0.6, right=0.98, bottom=0.45, wspace=0.02, hspace=0.05, top=0.85)
                
                if plot_antenna:
                
                    gs3=GS(1,1)
                    gs3.update(top= 0.35, left=0.6, right=0.98, bottom=0.05)
                    ax3=fig.add_subplot(gs3[0])
                    ax3.plot(M['ant']['x'], M['ant']['y'], 'o', ms=3, color=cm.Greys(0.5),zorder=1)
                
                    #if plot_weather:
                    if False:
#                    twind=15
                        
                    # wind rose
                        wind_rose = np.array([np.sin(np.radians(winddir)), np.cos(np.radians(winddir))])*windspeed
                    
                    # length of the wind arrow (in time)
                    # (taking the max of [x,y] axis extent, divide it by 3, and make a multiple of five out of it)
                        twind = np.round(np.max([np.diff(ax3.get_xlim()), np.diff(ax3.get_ylim())])/3./windspeed/5.)*5
                    
                        wind_rose *= twind
                        boxmin = np.array([ np.min(ax3.get_xlim()), np.min(ax3.get_ylim()) ]) + 10.
                    
                        xwind = wind_rose[0] if wind_rose[0] < 0 else 0.
                        ywind = wind_rose[1] if wind_rose[1] < 0 else 0.
                    
                        windmin = np.array([xwind, ywind])
                    
                        xy0 = boxmin - windmin
                        xy1 = wind_rose - windmin + boxmin
                    
                        xy = ( xy0 + xy1 ) /2.
                    
                        rotangle=np.mod((90-winddir)+90,180)-90
                        ax3.text(xy[0], xy[1], '%i s' % twind, transform=ax3.transData, ha='center', va='center', rotation=rotangle)
                        ax3.annotate('', xy=xy0, xytext=xy1, color=cm.Greys(0.5), size=20, arrowprops=dict(arrowstyle='fancy', fc=cm.hot(0.8), ec=cm.hot(0.6)), zorder=0)
                
            first=False
        
#        blcolor= cm.hsv((iplot-1.)/(nplot))
            blcolor=cmaps[iplot](0.8)
            blcolor2=cmaps[iplot](0.4)
        
            if plot_phase:
                iy2,ix2=divmod(iplot,nx2)
                ax2=fig.add_subplot(gs2[iplot])
                if bl_ant == 'MS':
                    if plot_original:
                        ax2.plot(data['DATA'][baseline][:,0], np.degrees(data['DATA'][baseline][:,1]), 'o', mew=0, mec=cm.Greys(0.6), ms=2., color=blcolor2, label='original')
                    if plot_corrected:
                        ax2.plot(data['CORRECTED_DATA'][baseline][:,0], np.degrees(data['CORRECTED_DATA'][baseline][:,1]), 'o', mew=0,  mec=cm.Greys(0.6), ms=2., color=blcolor, label='wvr corrected')
                else:
                    ax2.plot(data['CPARAM'][baseline][:,0], np.degrees(data['CPARAM'][baseline][:,1]), 'o', mew=0.1,  mec=cm.Greys(0.6), ms=2., color=blcolor, label='wvr correction')
            
                this_ylim = ax2.get_ylim()
                this_yrange = np.diff(this_ylim)
                if this_yrange < 360.:
                    ax2.set_ylim( this_ylim[0]-this_yrange/2., this_ylim[1]+this_yrange/2. )
                ax2.grid(color=cm.Greys(0.5))

#            ax2.yaxis.set_major_locator(MultipleLocator(90))
                ax2.xaxis.set_major_locator(MaxNLocator(5))

                if iy2 < ny2-1:
                    ax2.xaxis.set_ticklabels([])
                if ix2 > 0:
                    ax2.yaxis.set_ticklabels([])
            
                if (ix2 == 0) & (iy2 == ny2-1):
                    ax2.set_xlabel(r'$time$ (s)')
                    ax2.set_ylabel(r'Phase - $\phi$ (deg)')

                ax2.yaxis.set_minor_locator(aml())
                ax2.xaxis.set_minor_locator(aml())
        
            if plot_antenna:
            
                if bl_ant == 'MS':
                    ant1,ant2=baseline.split('-')
              
                    ax3.plot([antpos[ant1][0], antpos[ant2][0]], [antpos[ant1][1], antpos[ant2][1]], '-', lw=3, color=blcolor, zorder=1)
                    ax3.plot([antpos[ant1][0], antpos[ant2][0]], [antpos[ant1][1], antpos[ant2][1]], 'o', ms=5, color=cm.Greys(0.8), zorder=2)
                else:
                    ant1=baseline
#                
                    ax3.plot(antpos[ant1][0], antpos[ant1][1], 'o', ms=8, color=blcolor)

            if bl_ant == 'MS':
                label='%s (%.1fm)' % (baseline, blength)
            else:
                label='%s (ANT%i)' % (baseline, np.int(blength))

            if plot == 'BOTH':
            
                TPD_ratio = results['DATA'][baseline] / results['CORRECTED_DATA'][baseline]
                TPD_ratio[:,0] = results['CORRECTED_DATA'][baseline][:,0]
                ax1b.semilogx(TPD_ratio[:,0], TPD_ratio[:,1], '-', color=blcolor, lw=1.5)
            
            # original
                TPD = results['DATA'][baseline].copy()
                TPD[:,1] /= 10**(iplot/omag)
                ax1a.loglog(TPD[:,0], toPathlength(restfreq_val,TPD[:,1],verbose=False), 'o', label=label, color=blcolor2, ms=4, mew=0)
            
            # corrected
                TPD = results['CORRECTED_DATA'][baseline].copy()
                
                TPD[:,1] /= 10**((iplot+0.0)/omag)
                ax1a.loglog(TPD[:,0], toPathlength(restfreq_val,TPD[:,1],verbose=False), 'o', label=None, alpha=1., color=blcolor, ms=2., mew=0)
            
            # indicates the reduction factor 10**(iplot/omag)
                if iplot > 0 and np.any(np.isfinite(TPD[:,1])):
                    index = np.min(np.arange(len(TPD[:,1]))[np.isfinite(TPD[:,1])])
                    ax1a.text(TPD[0,0], toPathlength(restfreq_val,TPD[index,1],verbose=False), r'$\times10^{-%.1f}$' % (iplot/omag), transform=ax1a.transData, ha='center', va='bottom', rotation=70, color=blcolor, size=10)
            else:
                TPD = results[plot][baseline].copy()
                TPD[:,1] /= 10**(iplot/omag)
                ax1a.loglog(TPD[:,0],toPathlength(restfreq_val,TPD[:,1],verbose=False), 'o', alpha=1., label=label, color=blcolor, ms=4., mew=0.)
            
            # indicates the reduction factor 10**(iplot/omag)
                if iplot > 0 and np.any(np.isfinite(TPD[:,1])):
                    index = np.min(np.arange(len(TPD[:,1]))[np.isfinite(TPD[:,1])])
                    ax1a.text(TPD[0,0], toPathlength(restfreq_val,TPD[index,1],verbose=False), r'$\times10^{-%.1f}$' % (iplot/omag), transform=ax1a.transData, ha='center', va='bottom', rotation=70, color=blcolor, size=10)

            if iplot == nplot-1 or Iplot == Nplot-1:
                #ax1a.set_ylabel(r'$\sigma_\gamma(\tau)$ (radians)')
                ax1a.set_ylabel(r'Two-Point-Deviation - $\phi_\sigma(T)$ (microns)')

#            ax.grid(color=cm.Greys(0.5))
#            ax1a.set_xscale('log')
                if plot == 'BOTH':
                    ax1b.axhline(1, color='k')
                    #ax1b.set_xlabel(r'$\tau_0$ (s)')
                    ax1b.set_xlabel(r'T (sec)')

                    #ax1b.set_ylabel(r'$\frac{\sigma_\gamma(\tau)}{\sigma_{\gamma,WVR}(\tau)}$')
                    ax1b.set_ylabel(r'TPD Ratio - $\frac{\phi_\sigma(T)}{\phi_{\sigma,WVR}(T)}$')

                    ax1b.grid(color=cm.Greys(0.5))
                    if ylimB is not None:
                        ax1b.set_ylim( ylimB )
                    ax1a.xaxis.set_ticklabels([])
                else:
                    ax1a.set_xlabel(r'$\tau_0$ (s)')
                if xlim is not None:
                    ax1a.set_xlim( xlim )
                    if plot == 'BOTH':
                        ax1b.set_xlim( xlim )
                if ylim is not None:
                    ax1a.set_ylim( ylim )
            
                this_ylim = np.log10(ax1a.get_ylim())
                this_xlim = np.log10(ax1a.get_xlim())
            
                n_x = np.ceil( np.max(this_xlim) - np.min(this_xlim) )
                n_y = np.ceil( np.max(this_ylim) - np.min(this_ylim) )
            
                ax1a.grid(color=cm.Greys(0.5))
                for i in range(np.int(n_x+n_y)*4):
                    ax1a.plot( np.array(ax1a.get_xlim()), 10**(this_ylim[1] + n_x - i/4.) * np.array(ax1a.get_xlim())**-1, ':', color=cm.Greys(0.25))

                ax1a.legend(loc=8, bbox_to_anchor = (0.5, 1.05), numpoints=1, prop={'size':'small'}, ncol=2)
            
                if figdir is not None:
                    #print '{0}/ADEV{1:03d}.png'.format(figdir,inplot)
                
                    if plot_antenna:
                        ax3.set_aspect('equal')
                        ax3.set_ylabel('Y (m)')
                        ax3.set_xlabel('X (m)')
                        ax3.grid(color=cm.Greys(0.3))
                        ax3.xaxis.set_major_locator(MaxNLocator(5))
                        ax3.yaxis.set_major_locator(MaxNLocator(5))
                        ax3.xaxis.set_minor_locator(aml())
                        ax3.yaxis.set_minor_locator(aml())

                    if eps:
                        fig.savefig('{0}/eps/ADEV{1:03d}.eps'.format(figdir,inplot))
                    fig.savefig('{0}/ADEV{1:03d}.png'.format(figdir,inplot), dpi=300.)
                    inplot+=1
        
            # increase the counter
            iplot+=1
            Iplot+=1





    
def WVRScaleFactor(vis_run='',refant='',badant='',badbl='',toplot='ref',overwrite=False, verbose=False, use12=False):
    ''' This code reads the fits files created for a dataset (one source) 
    that has been processed in Allegro routines and compares a range of 
    scaling factors for the WVR solution and makes a plot of the output
    
    :param vis_run: the visibility dataset to read from - prefix WITHOUT .vis.fits extension (eg. uid___A002_Xad5116_X5dd)
    :type vis_run: string
    :param refant: reference antenna - no default - must supply
    :type refant: string
    :param badant: any bad antennas to flag out, comma separated (eg.  'DV01,DV02') - default = ''
    :type badant: string
    :param badbl: any bad baseline to flag out, comma separated (eg. 'DV01-DV02') - default = ''
    :type badbl: string
    :param toplot: comare baselines with the reference antenna (ref) - default -  or all baselines (all)
    :type toplot: string
    :param overwrite: overwrite the whole director - default = False
    :type overwrite: boolean
    :param verbose: do you want extened text output? - Default False (minimal text output)
    :type verbose: boolean
    
    
    :returns: List of the scaling factors and timescales
    :rtype: List Array
    '''


    results = 'fits/'+vis_run+'.vis.fits'
    results_u = 'fits/'+vis_run+'.viswvr.fits'

    # extra setup parameters
    meta=None
    data=None
    xlim=None
    ylim=None
    figdir='plots/'+vis_run+'/scale'
    eps=False
    figsize=[5.6, 7.5]
    nplot=2
    omag=1.


    # setup plots
       
    if toplot == 'ref':
        toplot='refant'


       # set up the parameters for the plots
    params = {'backend': 'eps',
              'axes.labelsize': 8,
              'font.size': 6,
              'legend.fontsize': 8,
              'xtick.labelsize': 6,
              'ytick.labelsize': 6,
              'font.family': 'sans',
              }
    # color parameter maps
    cmaps=[cm.Greys, cm.Blues, cm.Greens, cm.Reds, cm.Purples, cm.Oranges]
    pl.rcParams.update(params)

    # this is figure location 

    if figdir is not None:
        if os.path.exists(figdir) and overwrite:
            pa = figdir.split('/')
            if len(pa) > 1:
                os.chdir(Join(pa[:-1],'/'))   # change to directory
                os.system('rm -rf %s' % pa[-1]) # removes scale ?
                os.chdir(Join(np.repeat('..',len(pa)-1),'/')) # moves back
            else:
                os.system('rm -rf %s' % figdir) # this removed and remakes - but above does first
                os.system('mkdir -p %s' % figdir)
            if eps:   ## FIXED TO FALSE 
                os.system('mkdir -p %s/eps' % figdir)
        if not os.path.exists(figdir):    # as deleted it remakes
            os.system('mkdir -p %s' % figdir)
            if eps:
                os.system('mkdir -p %s/eps' % figdir)


    # set due to redundant read inputs 
        
    pwv='pwv: nan'
    restfreq=-1
    intent=''
    spw=''
    band=0
    pwvval=-1.

    ## ok so this is the read in of the ms (vis), as specified
    ## we also want the viswvr which is the unity

    # always run this
    print ''
    print 'Running WVR scaling optimisation for'
    print vis_run
    print ''


    if verbose:
        print ''
        print 'Reading in the data - ready for scaling and then plotting'
        print ''
     
    if type(results) is str:
        try:
            uid,data_type,suffix=results.split('.')
        except ValueError:
            try:
                uid,name_addition,data_type,suffix=results.split('.')
                uid=uid+'.'+name_addition
            except ValueError:
                try:
                    uid,name_addition1,name_addition2,data_type,suffix=results.split('.')
                    uid=uid+'.'+name_addition1+'.'+name_addition2
                except ValueError:
                    uid,name_addition1,name_addition2,name_addition3,data_type,suffix=results.split('.')
                    uid=uid+'.'+name_addition1+'.'+name_addition2+'.'+name_addition3



            


        d=fits.open(results)
        pwvval=d[0].header['PWV']
        if pwvval > 0:
            pwv='pwv: %.2f mm' % pwvval
        
        restfreq='f: %.1f GHz' % (d[0].header['RESTFREQ']*1e-9)
        bl_ant=d[0].header['ALLAN'].strip().upper()
        spw='spw: %i' % d[0].header['SPW']
        band=d[0].header['BAND']
    
        toplabel='Band%i, %s' % (band,pwv) 
        headlabel=Join([uid,restfreq],' - ')
#        %s   --- freq=%s    ---   pwv=%s
        # get the conversion:
        
        if bl_ant == 'MS':
            conv = {'ADEV_CORR': ['results','CORRECTED_DATA'],
                    'ADEV': ['results','DATA'],
                    'PHASE_CORR': ['data','CORRECTED_DATA'],
                    'PHASE': ['data','DATA']}
        else:
            conv = {'ADEV_CORR': ['results', 'CPARAM'],
                    'PHASE_CORR': ['data', 'CPARAM']}
        
        # metadata
        M={}
        
        # query all extensions
        extensions=['']
        for i in range(1,len(d)):
            extensions.append( d[i].header['EXTNAME'] )
        
        results={}
        data={}
        
        # antenna information required here - prob not?
        plot_antenna = False
        plot_weather = False
        plot_corrected = False
        plot_original = False
        
        for i,val in enumerate(extensions):
        
            D = d[i].data
            
            if val == 'ANT':
                plot_antenna = True  ## dont need
                M['ant'] = {}
                antenna_data = D.copy()
                M['ant']['name'] = antenna_data['name']
                M['ant']['x'] = antenna_data['x']
                M['ant']['y'] = antenna_data['y']
            elif val == 'WEATHER':
                plot_weather = True  ## dont need
                weather_data = D.copy()
        
            elif val == 'BL':
                M['bl'] = {}
                bl_data = D.copy()
                M['bl']['name'] = bl_data['name']
                M['bl']['xy'] = bl_data['xy']
                M['bl']['uv'] = bl_data['uv']
            elif val == 'PHASE_RMS':
                M['rms'] = {}
                phase_data=D.copy()
                M['rms']['name']=phase_data['name']
                M['rms']['phase']=phase_data['phase_rms']
                M['rms']['corr']=phase_data['corr_rms']
            elif val in conv.keys():
                if '_CORR' in val:
                    plot_corrected = True  ## options for later - dont need
                else:
                    plot_original = True  ## options for later - dont need
                todo=conv[val]
                if todo[0] == 'results':
                    results[todo[1]]={}
                    upd = results[todo[1]]
                else:
                    data[todo[1]]={}
                    upd = data[todo[1]]
            
                xunit = D.names[0]
                for col in D.names[1:]:
                    upd[col]=np.array([D[xunit],D[col]]).transpose()
        
        meta = M  
        # so this has set all the data for the raw phases


    if type(results_u) is str:  #  also get the info for the unity file
        
        #uid,data_type,suffix=results_u.split('.')  ## what will be read in?? - say ms only to keep same code in the go.py script
    #        toplabel=results.split('.')[0]
        try:
            uid,data_type,suffix=results_u.split('.')
        except ValueError:
            try:
                uid,name_addition,data_type,suffix=results_u.split('.')
                uid=uid+'.'+name_addition
            except ValueError:
                try:
                    uid,name_addition1,name_addition2,data_type,suffix=results_u.split('.')
                    uid=uid+'.'+name_addition1+'.'+name_addition2
                except ValueError:
                    uid,name_addition1,name_addition2,name_addition3,data_type,suffix=results_u.split('.')
                    uid=uid+'.'+name_addition1+'.'+name_addition2+'.'+name_addition3



        d_u=fits.open(results_u)
    
        bl_ant_u=d[0].header['ALLAN'].strip().upper() ##for data type    
        
        if bl_ant_u == 'MS':
            conv_u = {'ADEV_CORR': ['results','CORRECTED_DATA'],
                      'ADEV': ['results','DATA'],
                      'PHASE_CORR': ['data','CORRECTED_DATA'],
                      'PHASE': ['data','DATA']}
        else:
            conv_u = {'ADEV_CORR': ['results', 'CPARAM'],
                      'PHASE_CORR': ['data', 'CPARAM']}
       
        # metadata - this is different to get meta before
        M_u={}
        
        # query all extensions
        extensions_u=['']
        for i in range(1,len(d_u)):
            extensions_u.append( d_u[i].header['EXTNAME'] )
        
        results_u={}
        data_u={}
        
        # antenna information required here - prob not?
        plot_antenna = False
        plot_weather = False
        plot_corrected = False
        plot_original = False
        
        for i,val in enumerate(extensions_u):
        
            D_u = d_u[i].data
            
            if val == 'ANT':
                plot_antenna_u = True  ## dont need
                M_u['ant'] = {}
                antenna_data_u = D_u.copy()
                M_u['ant']['name'] = antenna_data_u['name']
                M_u['ant']['x'] = antenna_data_u['x']
                M_u['ant']['y'] = antenna_data_u['y']
            elif val == 'WEATHER':
                plot_weather_u = True  ## dont need
                weather_data_u = D_u.copy()
        
            elif val == 'BL':
                M_u['bl'] = {}
                bl_data_u = D_u.copy()
                M_u['bl']['name'] = bl_data_u['name']
                M_u['bl']['xy'] = bl_data_u['xy']
                M_u['bl']['uv'] = bl_data_u['uv']
            elif val == 'PHASE_RMS':
                M_u['rms'] = {}
                phase_data_u=D_u.copy()
                M_u['rms']['name']=phase_data_u['name']
                M_u['rms']['phase']=phase_data_u['phase_rms']
                M_u['rms']['corr']=phase_data_u['corr_rms']
            elif val in conv_u.keys():
                if '_CORR' in val:
                    plot_corrected_u = True  ## options for later - dont need
                else:
                    plot_original_u = True  ## options for later - dont need
                todo_u=conv_u[val]
                if todo_u[0] == 'results':
                    results_u[todo_u[1]]={}
                    upd_u = results_u[todo_u[1]]
                else:
                    data_u[todo_u[1]]={}
                    upd_u = data_u[todo_u[1]]
            
                xunit_u = D_u.names[0]
                for col in D_u.names[1:]:
                    upd_u[col]=np.array([D_u[xunit_u],D_u[col]]).transpose()
        
        meta_u = M_u    
        # so this is not set all the stuff also for the viswvr


    #    generate from the meta data now
    if 'ant' in meta.keys():
        plot_antenna=True  ## do we need ??
        antpos={}
        for iname, ix, iy in zip(M['ant']['name'], M['ant']['x'], M['ant']['y']):
            antpos[iname]=[ix,iy]
            
        # sort in projected baseline ->  correct is uv 
    blsort = sorted(zip(M['bl']['uv'], M['bl']['name'])) 


    ## other default plot setups from 

    if xlim is None:
        xlim=[ np.nan, np.nan ]
    if ylim is None:
        ylim=[ np.nan, np.nan ]
    ylimB=None

   

    iplot=0 # counter for the curves per page
    Iplot=0 # counter for the total number of curves
    Nplot=len(blsort) # number of plots
    inplot=1
    first=True

    # loop over the baseline sorted by length
    """
    Option 1 : Ref ant only baseline
    Sort by baseline and plot scales
    blenght = baseline length
    baseline = baseline name (Ant1-Ant2)
    
    Option 2 : All baselines
    Sort by baseline and plot scales
    blenght = baseline length
    baseline = baseline name (Ant1-Ant2)
    automatic if no reference antenna specified 
    """

    # always print this
    print ''
    print 'Data read...now selecting on the scaling to run'
    print ''

          
          
    # create list for the bad antennas
    flagants = badant.split(',')
    flagbls = badbl.split(',')  

    # TPD times 
    TPD_times = [6.0,12.0,32.0,64.0]
    Coh_times = [30.0,60.0,90.0,120.0]

    # loop over and store a dictionary
    scale_dict = {}
    scale_dict['bl_len'] = []
    scale_dict['bl_name'] = []
    scale_dict['bl_scale'] = [[],[],[],[]] # for 6,12,32,64 - TDP_times
    scale_dict['scale_final']=[[],[],[],[]] 
    scale_dict['scale_final_err']=[[],[],[],[]] 

    scale_dict['ratio']=[[],[],[],[]] # 6 and 12 only - normal and scaled  
          
    # Coherence values use RMS 
    # 4 for times 30,60,90,120  AND ONE for the entire time - Coh_times
    scale_dict['Cohraw']=[[],[],[],[],[]]
    scale_dict['Cohwvr']=[[],[],[],[],[]]
    scale_dict['Cohscaled']=[[],[],[],[],[]]

    if toplot =='refant' and len(refant)>2:
    ## do the refant calc only - make sure refant is set - if not will do all
        print ' Reference antenna bases only, using  %s '%refant

        # add to the bad flags so it doesnt use baselines without the refant
        for bl_one in M['bl']['name']:
            if refant != bl_one.split('-')[0] and refant != bl_one.split('-')[1]:
                flagbls.append(bl_one)
    else:
        print ' Refant not specified, or specified incorrectly'
        print ' will run all baseline analysis'
        toplot ='all'


    for blength, baseline in blsort:
       

        ## do checks here for bad baselines and ants 
        bl_combo1 = baseline.split('-')[0]
        bl_combo2 = baseline.split('-')[1]

        if baseline not in flagbls and bl_combo1 not in flagants and bl_combo2 not in flagants:
            
                # set the phases and time
            if verbose:
                print baseline
            T = data['DATA'][baseline][:,0]
            P = data['DATA'][baseline][:,1]
            U = data_u['CORRECTED_DATA'][baseline][:,1] # correction is applied to the unity file - no DATA
            W = data['CORRECTED_DATA'][baseline][:,1] # the WVR corrected data from CASA

            ## Data with bad WVR will have WVR = Phase in some cases
            ## check and exclude automatically

   
            if not np.mean(np.array(P) - np.array(W)) == 0.0 and len(np.array(P)[np.isfinite(P)]) > 1:

          
     
                if np.std(W - (U+P)) > 0.1:  # sometimes a bad bl with unwrap  - so dont use unity, use WVR corrected minus raw 
                # simply remake the unity i.e. W-P should = U
                    if verbose:
                        print 'baseline Unity different wrap.....reset using WVR - PHASES - no issue'
                    U = W - P


            

                # for the usable data keep it 
                # use the minimisation routine now
                for idn,tm in enumerate(TPD_times):
                    if len(P[np.isfinite(P)]) > 1:
                        scale_dict['bl_scale'][idn].append(WVRSminScaling(T,P,U,W,tm))
                    else:
                        scale_dict['bl_scale'][idn].append(np.nan)

            
                scale_dict['bl_len'].append(blength)
                scale_dict['bl_name'].append(baseline)
            
         
            
    # so now this is the list for all baselines and averages them
    # all have same weighting 


    for idn in range(len(TPD_times)):
        scale_dict['scale_final'][idn].append(np.mean(scale_dict['bl_scale'][idn]))
        scale_dict['scale_final_err'][idn].append(np.std(scale_dict['bl_scale'][idn]))
        # flagged antennas should return a nan so wont average correctly


      
    # find the stats, TPD and RMS for Coherence calcs

    for blength, baseline in blsort:
        ## do checks here for bad baselines and ants
        bl_combo1 = baseline.split('-')[0]
        bl_combo2 = baseline.split('-')[1]

        if baseline not in flagbls and bl_combo1 not in flagants and bl_combo2 not in flagants:

            # set the phases and time
            #print baseline
            T = data['DATA'][baseline][:,0]
            P = data['DATA'][baseline][:,1]
            U = data_u['CORRECTED_DATA'][baseline][:,1] # correction is applied to the unity file - no DATA
            W = data['CORRECTED_DATA'][baseline][:,1] # the WVR corrected data from CASA
            
            if not np.mean(np.array(P) - np.array(W)) == 0.0 and len(np.array(P)[np.isfinite(P)]) > 1 :

                if np.std(W - (U+P)) > 0.1:  
                    U = W - P


                  
                # first 6 sec values, then 12 sec 
                diff_time = np.median(np.diff(T))  # integration time
                value_use = [np.int(np.round(6.0/diff_time)),np.int(np.round(12.0/diff_time))]

                # calculate the phase streams -WVR correted and WVR scaled corrected - at 6sec
                phase_comp6 = P + scale_dict['scale_final'][0][0]*U  # uses 6 sec scaing value
                phase_corr6 = P + U
                     
                # the stats
                tps_raw6 = WVRS_calc2PtT(P,T,value_use[0])
                tps_corr6 = WVRS_calc2PtT(phase_corr6,T,value_use[0])
                tps_scaled6 = WVRS_calc2PtT(phase_comp6,T,value_use[0])
                    
                scale_dict['ratio'][0].append(tps_raw6[:,1]/tps_corr6[:,1])
                scale_dict['ratio'][1].append(tps_raw6[:,1]/tps_scaled6[:,1])
             
                # phases at 12 sec
                phase_comp12 = P + scale_dict['scale_final'][1][0]*U  # uses the 12 sec scaling value
                phase_corr12 = P + U
                          
                # the stats
                tps_raw12 = WVRS_calc2PtT(P,T,value_use[1])
                tps_corr12 = WVRS_calc2PtT(phase_corr12,T,value_use[1])
                tps_scaled12 = WVRS_calc2PtT(phase_comp12,T,value_use[1])

                scale_dict['ratio'][2].append(tps_raw12[:,1]/tps_corr12[:,1])
                scale_dict['ratio'][3].append(tps_raw12[:,1]/tps_scaled12[:,1])
                    
                # the RMS now for the coherence plots
            
                value_use_rms = np.round(np.array(Coh_times)/diff_time)
 
                # this is the coherence values for all baselines (ordered) based on the rms input at all 4 times 30,60,90,120s

                # use 6 second timescale by default - unless 12 sec chosen

                if not use12:
                    for idn in range(len(Coh_times)): # this will not do the last idn - as this is for the STDall
                        scale_dict['Cohraw'][idn].append(np.exp(-1*((STDC(P,np.int(value_use_rms[idn])))**2)/2))
                        scale_dict['Cohwvr'][idn].append(np.exp(-1*((STDC(phase_corr6,np.int(value_use_rms[idn])))**2)/2))
                        scale_dict['Cohscaled'][idn].append(np.exp(-1*((STDC(phase_comp6,np.int(value_use_rms[idn])))**2)/2))
                    # for RMS all calc over full length of BP observation
                    scale_dict['Cohraw'][len(Coh_times)].append(np.exp(-1*((STDall(P))**2)/2))
                    scale_dict['Cohwvr'][len(Coh_times)].append(np.exp(-1*((STDall(phase_corr6))**2)/2))
                    scale_dict['Cohscaled'][len(Coh_times)].append(np.exp(-1*((STDall(phase_comp6))**2)/2))
                else:
                    for idn in range(len(Coh_times)): # this will not do the last idn - as this is for the STDall
                        scale_dict['Cohraw'][idn].append(np.exp(-1*((STDC(P,np.int(value_use_rms[idn])))**2)/2))
                        scale_dict['Cohwvr'][idn].append(np.exp(-1*((STDC(phase_corr12,np.int(value_use_rms[idn])))**2)/2))
                        scale_dict['Cohscaled'][idn].append(np.exp(-1*((STDC(phase_comp12,np.int(value_use_rms[idn])))**2)/2))
                    # for RMS all calc over full length of BP observation
                    scale_dict['Cohraw'][len(Coh_times)].append(np.exp(-1*((STDall(P))**2)/2))
                    scale_dict['Cohwvr'][len(Coh_times)].append(np.exp(-1*((STDall(phase_corr12))**2)/2))
                    scale_dict['Cohscaled'][len(Coh_times)].append(np.exp(-1*((STDall(phase_comp12))**2)/2))


          
    # now begin the plot 
    figsize=np.array(figsize)

    pl.close(1)

    
    fig=pl.figure(1, figsize=figsize)
    fig.clf()
    # Setting up the head lines
    hcol=cm.Purples
    ax0=fig.add_axes([0., 0.93, 1., 0.07], axisbg=hcol(0.5))
    ax0.text(0.03, 0.1, toplabel, fontsize=18, color='w', transform=ax0.transAxes)
    # manual set of data type - to scaling
    data_type = 'scaling'
    ax0.text(0.97, 0.1, data_type, fontsize=25, color='w', transform=ax0.transAxes, ha='right', va='bottom')                
    ax0.xaxis.set_ticks([])
    ax0.yaxis.set_ticks([])
    #ax0.axis('off')
    ax0=fig.add_axes([0., 0.89, 1., 0.04], axisbg=hcol(0.8))
    
    ax0.text(0.03, 0.1, headlabel, fontsize=8, color='w', transform=ax0.transAxes)
    ax0.xaxis.set_ticks([])
    ax0.yaxis.set_ticks([])
    # header setup

    # Now the figures, here want 6 figures

    # left ones
    gs1=GS(3,1) #can also add height ratios in the plots eg , heighe_ratios=[2.5,1]
    ax1a=fig.add_subplot(gs1[0])
    ax1b=fig.add_subplot(gs1[1])
    ax1c=fig.add_subplot(gs1[2])
    gs1.update( left=0.10, right=0.48, hspace=0.02, bottom=0.05, top=0.82)

    # right ones
    gs2=GS(3,1)
    ## sub plots added when plotting in plot allan - not fixed depending on params - these are
    ax2a=fig.add_subplot(gs2[0])
    ax2b=fig.add_subplot(gs2[1])
    ax2c=fig.add_subplot(gs2[2])

    gs2.update( left=0.57, right=0.96, hspace=0.02, bottom=0.05, top=0.82)


    alp = 1.0 # always plot these as solid

    
    # left TPDs 6 and 32
    ax1a.plot(scale_dict['bl_len'],scale_dict['bl_scale'][0],marker='o',linestyle='',c='m',alpha=alp)
    ax1b.plot(scale_dict['bl_len'],scale_dict['bl_scale'][2],marker='o',linestyle='',c='m',alpha=alp)
        
    ax1a.grid(color=cm.Greys(0.5))
    ax1b.grid(color=cm.Greys(0.5))

    # scale all scaling plots the same
    max_ax1 =np.max(scale_dict['bl_scale'])
    min_ax1 =np.min(scale_dict['bl_scale'])
    
    ax1a.set_ylim(min_ax1-0.05,max_ax1+0.1)
    ax1b.set_ylim(min_ax1-0.05,max_ax1+0.1)

    ax1a.set_xlim(0,np.max(scale_dict['bl_len'])+100)
    ax1b.set_xlim(0,np.max(scale_dict['bl_len'])+100)

    # now the bl averaged value
    ax1a.plot([0,np.max(scale_dict['bl_len'])+100],[scale_dict['scale_final'][0][0],scale_dict['scale_final'][0][0]],linestyle='--',c='k')
    ax1b.plot([0,np.max(scale_dict['bl_len'])+100],[scale_dict['scale_final'][2][0],scale_dict['scale_final'][2][0]],linestyle='--',c='k')

   
    # need to annotate
    ax1b.set_xlabel('Baseline (m)')
    ax1b.set_ylabel('Optimal Scaling Factor')
    ax1b.yaxis.set_label_coords(-0.13,1.0,transform=None)
    ax1a.annotate('Scaling values per baseline',xy=(0,1),xycoords='axes fraction', xytext=(0.01,1.02))
    
    ax1a.annotate('6 sec',xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.94))
    ax1b.annotate('32 sec',xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.94))


   
    ax1a.annotate('%s +/- %s'%(str(np.round(scale_dict['scale_final'][0][0],2)),str(np.round(scale_dict['scale_final_err'][0][0],2))),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.90))
    ax1b.annotate('%s +/- %s'%(str(np.round(scale_dict['scale_final'][2][0],2)),str(np.round(scale_dict['scale_final_err'][2][0],2))),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.90))


    # and the right TPDs
    ax2a.plot(scale_dict['bl_len'],scale_dict['bl_scale'][1],marker='o',linestyle='',c='m',alpha=alp)
    ax2b.plot(scale_dict['bl_len'],scale_dict['bl_scale'][3],marker='o',linestyle='',c='m',alpha=alp)
       
    ax2a.grid(color=cm.Greys(0.5))
    ax2b.grid(color=cm.Greys(0.5))
         
    ax2a.set_ylim(min_ax1-0.05,max_ax1+0.1)
    ax2b.set_ylim(min_ax1-0.05,max_ax1+0.1)

    ax2a.set_xlim(0,np.max(scale_dict['bl_len'])+100)
    ax2b.set_xlim(0,np.max(scale_dict['bl_len'])+100)

    # now the final line
    ax2a.plot([0,np.max(scale_dict['bl_len'])+100],[scale_dict['scale_final'][1][0],scale_dict['scale_final'][1][0]],linestyle='--',c='k')
    ax2b.plot([0,np.max(scale_dict['bl_len'])+100],[scale_dict['scale_final'][3][0],scale_dict['scale_final'][3][0]],linestyle='--',c='k')

    # need to annotate
    ax2c.set_xlabel('Baseline (m)')
    ax2c.set_ylabel('Phase TPD Improvement')

    
    ax2a.annotate('%s +/- %s'%(str(np.round(scale_dict['scale_final'][1][0],2)),str(np.round(scale_dict['scale_final_err'][1][0],2))),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.90))
    ax2b.annotate('%s +/- %s'%(str(np.round(scale_dict['scale_final'][3][0],2)),str(np.round(scale_dict['scale_final_err'][3][0],2))),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.90))
        
    ax2a.annotate('12 sec',xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.94))
    ax2b.annotate('64 sec',xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.94))
    
    # the ratio and coherence plots
    ax2c.plot([0,np.max(scale_dict['bl_len'])+100],[1,1],marker='',linestyle='--',c='k')
    
    # the WVR scaled data at 6 sec / WVR normal at 6 sec - or 12 sec 
    if not use12:
        ax2c.plot(scale_dict['bl_len'],np.array(scale_dict['ratio'][1])/np.array(scale_dict['ratio'][0]),marker='',linestyle='-',c='m',alpha=alp,linewidth=3.0)
    else:
        ax2c.plot(scale_dict['bl_len'],np.array(scale_dict['ratio'][3])/np.array(scale_dict['ratio'][2]),marker='',linestyle='-',c='m',alpha=alp,linewidth=3.0)


    # plot the average coherence with baseline for full bandpass observation

    ax1c.plot(scale_dict['bl_len'],scale_dict['Cohraw'][len(Coh_times)],marker='',linestyle='-.',c='m',alpha=alp,linewidth=3.0)
    ax1c.plot(scale_dict['bl_len'],scale_dict['Cohwvr'][len(Coh_times)],marker='',linestyle='--',c='m',alpha=alp,linewidth=3.0)
    ax1c.plot(scale_dict['bl_len'],scale_dict['Cohscaled'][len(Coh_times)],marker='',linestyle='-',c='m',alpha=alp,linewidth=3.0)

 
    
    

    ax1c.set_xlabel('Baseline (m)')
    ax1c.set_ylabel('Coherence')

    ax2c.grid(color=cm.Greys(0.5))
    ax1c.set_xlim(0,np.max(scale_dict['bl_len'])+100)
    ax2c.set_xlim(0,np.max(scale_dict['bl_len'])+100)
    ax1c.grid(color=cm.Greys(0.5))

               
    ax2c.set_ylim(0.5,1.5) # ratio of improvement of 6 sec statistic
    ax1c.set_ylim(-0.2,1.0) # the coherence values

    ## Set visible or not for the axes
    ax1a.xaxis.set_ticklabels([])
    ax2a.xaxis.set_ticklabels([])
    ax1b.xaxis.set_ticklabels([])
    ax2b.xaxis.set_ticklabels([])

    if not use12:
        ax2c.annotate('Improvement of TPD at 6s = ',xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.08))      
    else:
        ax2c.annotate('Improvement of TPD at 12s = ',xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.08))      

    ax1c.annotate('Coherence Improvement (Scaled WVR vs. Std):',xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.12))
            
    # calc for annotation and summary results 

    # Longer than 10% of the max baseline
    bl_long = np.where(np.array(scale_dict['bl_len']) > np.max(scale_dict['bl_len'])*0.1)[0]
   
    # have to do in this 2 step
    Coh_ratio_full = np.array(scale_dict['Cohscaled'])/np.array(scale_dict['Cohwvr']) # list of 5 lists 

    Coh_ratio_long = [np.mean(Coh_ratio_full[0][bl_long]),np.mean(Coh_ratio_full[1][bl_long]),np.mean(Coh_ratio_full[2][bl_long]),np.mean(Coh_ratio_full[3][bl_long]),np.mean(Coh_ratio_full[4][bl_long])]

    Coh_ratio = np.mean(np.array(scale_dict['Cohscaled'])/np.array(scale_dict['Cohwvr']),axis=1)

   


    #ax1c.annotate('%s over the entire BP observation'%(str(Coh_ratio[4])[0:4]),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.08))  # need the 4 values
    #ax1c.annotate('%s over a 60s interval'%(str(Coh_ratio[1])[0:4]),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.04))  # need the 4 values

   
    ax1c.annotate('%s over the entire BP observation'%(str(np.round(Coh_ratio[4],2))),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.08))  # need the 4 values
    ax1c.annotate('%s over a 60s interval'%(str(np.round(Coh_ratio[1],2))),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.04))  # need the 4 values

    #ax1c.annotate('%s over 30s, %s over 60s'%(str(Coh_ratio[0])[0:4],str(Coh_ratio[1])[0:4]),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.08))  # need the 4 values
    #ax1c.annotate('%s over 90s, %s over 120s'%(str(Coh_ratio[2])[0:4],str(Coh_ratio[3])[0:4]),xy=(0,1),xycoords='axes fraction',xytext=(0.02,0.04))  # need the 4 values

    # line for key
    ax1c.plot([100,1000],[0.078,0.078],marker='',linestyle='-.',c='m',alpha=alp,linewidth=3.0)
    ax1c.plot([100,1000],[0.115,0.115],marker='',linestyle='--',c='m',alpha=alp,linewidth=3.0)
    ax1c.plot([100,1000],[0.15,0.15],marker='',linestyle='-',c='m',alpha=alp,linewidth=3.0)
    ax1c.annotate('Raw',xy=(0,1),xycoords='axes fraction',xytext=(0.21,0.19))
    ax1c.annotate('Standard WVR',xy=(0,1),xycoords='axes fraction',xytext=(0.21,0.23))
    ax1c.annotate('Scaled WVR',xy=(0,1),xycoords='axes fraction',xytext=(0.21,0.28))

    Imp_ratio = [np.mean(np.array(scale_dict['ratio'][1])/np.array(scale_dict['ratio'][0])),np.mean(np.array(scale_dict['ratio'][3])/np.array(scale_dict['ratio'][2]))]
    if not use12:
        ax2c.annotate('%s'%str(np.round(Imp_ratio[0],2)),xy=(0,1),xycoords='axes fraction',xytext=(0.58,0.08))
    else:
        ax2c.annotate('%s'%str(np.round(Imp_ratio[1],2)),xy=(0,1),xycoords='axes fraction',xytext=(0.58,0.08))

    # set a plotting suffix and the print out (NEW)
    if toplot == 'refant':
        plot_suff = str(toplot+'_'+refant)
   
    else:
    
        plot_suff = str(toplot+'_baselines') # default for all - different image name 
      

    print ''
    print ' ** Output overview **'
    print ''
    print ' Scaling factors using 6, 12, 32 and 64 sec TDP timescales are: '
    print ' 6 sec',str(np.round(scale_dict['scale_final'][0][0],2)),'+/-',str(np.round(scale_dict['scale_final_err'][0][0],2))
    print ' 12 sec',str(np.round(scale_dict['scale_final'][1][0],2)),'+/-',str(np.round(scale_dict['scale_final_err'][1][0],2))#str(scale_dict['scale_final'][1][0])[0:4],'+/-',str(scale_dict['scale_final_err'][1][0])[0:4]
    print ' 32 sec',str(np.round(scale_dict['scale_final'][2][0],2)),'+/-',str(np.round(scale_dict['scale_final_err'][2][0],2))#str(scale_dict['scale_final'][2][0])[0:4],'+/-',str(scale_dict['scale_final_err'][2][0])[0:4]
    print ' 64 sec',str(np.round(scale_dict['scale_final'][3][0],2)),'+/-',str(np.round(scale_dict['scale_final_err'][3][0],2))#str(scale_dict['scale_final'][3][0])[0:4],'+/-',str(scale_dict['scale_final_err'][3][0])[0:4]
    print ' Phase Two-Point-Deviation improvement at 6s is',str(np.round(Imp_ratio[0],2))
    print ' Phase Two-Point-Deviation improvement at 12 is',str(np.round(Imp_ratio[1],2))
    print ''
    if not use12:
        print ' ** Using the 6 second scaling values **'
        print ''
        scale_touse = (np.round(scale_dict['scale_final'][0][0],2))
    else:
        print ' ** Using the 12 second scaling values **'
        print ''
        scale_touse = (np.round(scale_dict['scale_final'][1][0],2))

    print ' The Coherence improvement (scaled WVR vs. standard WVR calibration)'
    print ' is %s over the entire observation of the bandpass'%(str(np.round(Coh_ratio[4],2)))
    print ' and are %s over 30s, %s over 60s, %s over 90s, %s over 120s'%(str(np.round(Coh_ratio[0],2)),str(np.round(Coh_ratio[1],2)),str(np.round(Coh_ratio[2],2)),str(np.round(Coh_ratio[3],2)))#%(str(Coh_ratio[0])[0:4],str(Coh_ratio[1])[0:4],str(Coh_ratio[2])[0:4],str(Coh_ratio[3])[0:4])
    print ''
    print ' Considering only baselines longer than 10% of Bmax (excluding the short baselines) '
    print ' the improvement in the coherence over the Bandpass observations is %s '%(str(np.round(Coh_ratio_long[4],2)))
    print ' and are %s over 30s, %s over 60s, %s over 90s, %s over 120s'%(str(np.round(Coh_ratio_long[0],2)),str(np.round(Coh_ratio_long[1],2)),str(np.round(Coh_ratio_long[2],2)),str(np.round(Coh_ratio_long[3],2)))
    print ''

    # extra print of recommended useage or not
    # based on the full observations 
    if Imp_ratio[0] > 1.05:
        print ' The TPD statistic of the phases is improved by over 5% on short timescales'
    elif  Imp_ratio[0] < 1.05:
        print ' The TPD statistic of the phases are NOT improved by more than 5%'
                

    if np.max(Coh_ratio) > 1.01 and np.max(Coh_ratio) < 1.05:
        print ' and the Coherence over the bandpass observation is improved up to 5%,'
        print ' there maybe some small improvement in the science image dependent on'
        print ' the cycle time -  targets with an on-source scan time similar to the'
        print ' bandpass length can expect up to the same improvement - for shorter'
        print ' on-source scan times note the expected improvement for the timescales above'
        print ''
        print ' Please edit the uid___A002_X........scriptForCalibration.py' 
        print ' to include the scale = '+str(scale_touse)+', in WVRGCAL'
    elif np.max(Coh_ratio) > 1.05:
        print ' and the Coherence over time is improved by more than 5%, such that'
        print ' there should be a noticable improvement in the science image dependent'
        print ' on cycle time - targets with an on-source scan time similar to the'
        print ' bandpass length can expect up to the same improvement - for shorter'
        print ' on-source scan times note the expected improvement for the timescales above'
        print ''
        print ' please edit the uid___A002_X........scriptForCalibration.py' 
        print ' to include: scale = '+str(scale_touse)+', in WVRGCAL'
    elif np.max(Coh_ratio) < 1.01 and Imp_ratio[0] > 1.05:
        print ' however, the Coherence over times are not improved although the small'
        print ' time phase statistics are - your observations were likely conducted'
        print ' in excellent conditions already with good coherence'
        print '- WVR scaling will not improve the data further -'
    elif np.max(Coh_ratio) < 1.01:
        print ' and the coherence is unchanged.'
        print ' Scaling the WVR will not improve your data'
        print ' compared to the standard WVR application'
        
    print ''
    print ' **'
    
    print ''
    print ' ** '
    print ' NOTE: If you appear to have erroneous results '
    print ' please check you have flagged all problem antennas '
    print ' ** ' 

     # save the figure
    if not use12:
        fig.savefig('{0}/Scaling_{1}_RMS_MINFNC_NEW_6s.png'.format(figdir,plot_suff), dpi=300.)
    else:
        fig.savefig('{0}/Scaling_{1}_RMS_MINFNC_NEW_12s.png'.format(figdir,plot_suff), dpi=300.)


    # make the return
    summary_values={}
    summary_values['data']=vis_run
    summary_values['6sec_scalefactor']=scale_dict['scale_final'][0][0]
    summary_values['12sec_scalefactor']=scale_dict['scale_final'][1][0]
    summary_values['32sec_scalefactor']=scale_dict['scale_final'][2][0]
    summary_values['64sec_scalefactor']=scale_dict['scale_final'][3][0]
    summary_values['phaseImp_ratio@6']=Imp_ratio[0]
    summary_values['phaseImp_ratio@12']=Imp_ratio[1]
    summary_values['CohImp30sec']=Coh_ratio[0]
    summary_values['CohImp60sec']=Coh_ratio[1]
    summary_values['CohImp90sec']=Coh_ratio[2]
    summary_values['CohImp120sec']=Coh_ratio[3]
    summary_values['CohImpAll']=Coh_ratio[4]
    summary_values['CohRawAve60']=np.mean(scale_dict['Cohraw'][1])
    summary_values['CohWVRAve60']=np.mean(scale_dict['Cohwvr'][1])
    summary_values['CohScaledAve60']=np.mean(scale_dict['Cohscaled'][1])
    summary_values['CohRawAveAll']=np.mean(scale_dict['Cohraw'][4])
    summary_values['CohWVRAveAll']=np.mean(scale_dict['Cohwvr'][4])
    summary_values['CohScaledAveAll']=np.mean(scale_dict['Cohscaled'][4])
    
   
    #summary_values=zip(times_out,values_out)
    return summary_values
