## this is the set of tools for the WVR scaling analysis
## Developed at Allegro, ALMA Arc node leiden

## Import required for the below to run and refrence correctly

import os
from wvrScaleImportFuncs import *
from recipes.almahelpers import tsysspwmap




def WVRcheck(vis):
	''' Check whether wvr calibration has already been applied to the dataset. This
	is done by checking the log history of the measurement set
	
	:param vis: measurement set
	:type vis: string
	:returns: WVR calibration applied?
	:rtype: boolean
	'''
	
	calibrated=False
	dirname='%s/HISTORY' % vis
	if os.path.exists(dirname):
		tb.open(dirname)
		for line in tb.getcol('MESSAGE'):
			if len(re.findall('gaintable.*wvr.*', line)) > 0:
				calibrated=True
		tb.close()
	return calibrated

def WVRcalib(vis, wvr, overwrite=False, reverse=False, toff=0., scale=1.0, tsystab=None):
	''' Calculate and apply the WVR table to an ms
	
	:param vis: measurement set
	:type vis: str
	:param wvr: WVR table
	:type wvr: str
	:param overwrite: Overwrite files?
	:type overwrite: boolean
	:param reverse: Use the reverse option in ``wvrgcal``?
	:type reverse: boolean
        :param scale: The scale value for wvrgcal
        :type scale: float 
        :param tsystab: an option Tsys table for the applycal step
	:type tsystab: tsys solution table
	'''
	
	if ( not os.path.exists(wvr) ) or ( os.path.exists(wvr) and overwrite ):
		os.system('rm -rf %s' % wvr)
		print "  * Doing wvr with reverse = ", reverse
		if reverse: wvrgcal(vis=vis, caltable=wvr, toffset=toff, reversespw='*',scale=scale)
		else: wvrgcal(vis=vis, caltable=wvr, toffset=toff, scale=scale)
	print "starting Applying the WVR solutions"
        if scale != 1.0:
            print ' with a WVR scaling factor of %s'%scale
	if tsystab is None:
		applycal(vis=vis, gaintable=wvr, flagbackup=False)
		print " Finished Applying the WVR solutions"
	else:
		# Need to do the Tsys map too and pass that 
		tsysmap = tsysspwmap(vis = vis, tsystable = tsystab, tsysChanTol = 1)
		applycal(vis=vis, gaintable=[tsystab,wvr], flagbackup=False,spwmap=[tsysmap,[]])
		print " Finished Applying the WVR and Tsys solutions"


def asdm_importToCASA(asdm, vis=None, intent='BANDPASS', verbose=False, scans_use=False, Tsysinc=False):
	''' Imports the ASDM into CASA. This function
	    es, which scans to include in the ms from the asdm
	    and then import only those scans.  Finally, select a single
	    correlation to import.
	    
	    :param asdm: ASDM name
	    :type asdm: string
	    :param vis: output vis name (if None, it defaults to asdm+'.ms'
	    :type vis: string
	    :param intent: imports only scans with that intent
	    :type intent: string
	    :param verbose: verbose
	    :type verbose: boolean
	    :param scans_use: manually select scans e.g. '0:3'
	    :type scans_use: string
	    :param Tsysinc: to import tsys - needed for images later
	    :type Tsysinc: boolean
	    :returns: True if import succeeded
	    :rtype: boolean
	    
	    
	'''

	if vis is None:
		vis = '%s.ms' % asdm

	#tmp_vis='%s_tmp' % vis

	# remove previous instances of files? required ?
	for directory in [vis]:
		if os.path.exists(directory):
			os.system('rm -rf %s' % directory)

	# importasdm doesn't allow to specify a single source for input.
	# I get around this by determining which scans belong to the source
	# I'm interested in extracting.
	if not scans_use:
		scans=asdm_getCalibScans(asdm, intent=intent)
	else:
		scans=scans_use
		

	
	if scans is not None:
		
		if not Tsysinc:
			if verbose: print " Importing Scans=%s" % scans[2:]
			importasdm(asdm=asdm, vis=vis, overwrite=True, scans=scans, asis='Antenna Station CalWVR CalAtmosphere Receiver Source')
		else:
			scans = '0:0~'+scans[-1] 
			if verbose: print " Importing Scans=%s" % scans[2:]
			importasdm(asdm=asdm, vis=vis, overwrite=True, scans=scans, asis='Antenna Station CalWVR CalAtmosphere Receiver Source')
		return True
	else:
		print "No Scans Found"
		return False


def asdm_getCalibScans(asdm, intent=None, field=None, combine='AND'):
	'''Finds the scans by intent
	
	:param asdm: ASDM name
	:type asdm: str
	:param intent: search for this intent
	:type intent: str
	:param field: search for scans on this field
	:type field: str
	:returns: string with scans
	:rtype: str
	
	'''

	if intent is None and field is None:
		print " You have to specify at least intent or field"
		return None

	if combine == 'AND' and intent is not None and field is not None:
		num_match = 2
	else:
		num_match = 1

	search_intent=True
	search_field=True
	if intent is None:
		search_intent=False
	if field is None:
		search_field=False

	# returns the scan_fields and scan_intents
	scans=[]
	xml=ET().parse('%s/Scan.xml' % asdm)
	for row in xml.findall('row'):
		append = [False, False]
		if search_intent:
			append[0] = True if intent in row.findtext('scanIntent') else False
		if search_field:
			append[1] = True if intent in row.findtext('sourceName') else False
		
		if sum(append) >= num_match:
			scans.append(row.findtext('scanNumber'))

	if len(scans) > 0:
		return '0:%s' % Join(scans,',')
	else:
		return None


def getLastSpwID_ms(vis):
	'''  Find which is the last averaged SPW which is used for scaling 
	     
	     :param vis: measurement set
	     :type vis: string
	     :returns: ID of the last channel-averaged spectral window
	     :rtype: int
	'''
	ms.open(vis)
	
	spwnum_holder = ms.getspectralwindowinfo()
	# the keys are the spw numbers
	# these are strings so make into an array
	#spwnum_list = spwnum_holder.keys()
	spwnum_arr = [int(i) for i in spwnum_holder] # pulls key automatically

	spwnum = np.max(spwnum_arr)
	
	# check it is an averaged channel
	checker = False
	while checker == False:
		if spwnum_holder[str(spwnum)]['NumChan'] and spwnum_holder[str(spwnum)]['TotalWidth'] > 888437500.0 :  # this is half of a 2GHz FDM
			checker=True
		else:
			spwnum = spwnum-1
		if spwnum == 0:
			print ' WARNING ... CANNOT FIND AVERAGED SPW...PIPE WILL BREAK'
			print ' You probably only have narrow bands '
			

	ms.close()
	return spwnum


def getScienceSpwID_ms(vis):
	'''  Find which are the science SPWs for split before imaging 
	     
	     :param vis: measurement set
	     :type vis: string
	     :returns: ID of the spectral windows
	     :rtype: string
	'''
	ms.open(vis)
	
	spwnum_holder = ms.getspectralwindowinfo()
	# the keys are the spw numbers
	# these are strings so make into an array
	#spwnum_list = spwnum_holder.keys()
	spwnum_arr = [int(i) for i in spwnum_holder] # pulls key automatically

	spwnum = np.max(spwnum_arr)
	
	# check it is an averaged channel
	checker = False
	while checker == False:
		if spwnum_holder[str(spwnum)]['NumChan'] and spwnum_holder[str(spwnum)]['TotalWidth'] > 888437500.0 :  # this is half of a 2GHz FDM
			checker=True
		else:
			spwnum = spwnum-1
		if spwnum == 0:
			print ' WARNING ... CANNOT FIND AVERAGED SPW...PIPE WILL BREAK'
			print ' You probably only have narrow bands '
			

	ms.close()
	return spwnum



def getband(freq):
	''' Identify the Band for specific frequency (in GHz)
	'''
	lo=np.array([0,0,84,125,157,211,275,385,602,787])*1e9
	hi=np.array([0,0,116,163,212,275,373,500,720,950])*1e9
	return np.arange(1,len(lo)+1)[(freq>lo)&(freq<hi)][0]


def antennaENU(vis, table='ANTENNA', position='POSITION', name='NAME'):
	''' convert antenna IRTF coordinates to local E(ast)N(orth)U(p)
	    coordinates 
	'''
	tb.open('%s/%s' % (vis,table))
	x,y,z=tb.getcol(position)
	if name is not None:
		n=tb.getcol(name)
	tb.close()

	r=np.sqrt(x**2+y**2+z**2)
	th=np.pi/2 - np.arccos(z/r)
	ph=np.arctan(y/x)

	cx,cy,cz,phi,lam=getCOFA(vis)
	cr=np.sqrt(cx**2+cy**2+cz**2)
	cth=np.pi/2 - np.arccos(cz/cr)
	cph=np.arctan(cy/cx)

	y=(th - cth)*r
	x=(ph - cph)*r*np.cos(th)
	z=r-cr
	
	if name is not None:
		return x,y,z,n
	else:
		return x,y,z

def stokes(nums):
	'''converts the integer value into a
	string representation of the Stokes parameter:
	 1 - I
	 2 - Q
	 3 - U
	 4 - V
	 5 - RR
	 6 - RL
	 7 - LR
	 8 - LL
	 9 - XX
	10 - XY
	11 - YX
	12 - YY
	13 - RX
	14 - RY
	15 - LX
	16 - LY
	17 - XR
	18 - XL
	19 - YR
	20 - YL
	21 - PP
	22 - PQ
	23 - QP
	24 - QQ
	'''
	
	pol=['', 'I', 'Q', 'U', 'V', 'RR', 'RL', 'LR', 'LL', 'XX', 'XY', 'YX', 
	     'YY', 'RX', 'RY', 'LX', 'LY', 'XR', 'XL', 'YR', 'YL', 'PP', 'PQ', 'QP', 'QQ']
	
	try:
		len(nums)
		p=[]
		for num in nums:
			p.append(pol[num])
	except TypeError:
		p=pol[num]
	
	return p


def mjd_to_unix(t, seconds=False):
	'''converts **Modified Julian Date** (MJD) to a
	unix timestamp.
	
	:param t: time
	:type t: float
	:param seconds: indicated if the time is given in seconds
	:type seconds: bool
	:returns: Unix timestamp (in seconds)
	
	Generally, the CASA output is MJD in seconds.
	
	>>> from datetime import datetime
	>>> t_unix = mjd_to_unix(4.9144e9, seconds=True)
	>>> t_unix
	1407683199.9999998
	>>> datetime.fromtimestamp(t_unix)
	datetime.datetime(2014, 8, 10, 17, 6, 40)
	
	'''
	
	if seconds:
		s=1/86400.
	else:
		s=1.
	return (np.array(t)*s-40587.0)*86400.
	


def unix_to_string(t, format='%Y/%m/%d/%H:%M:%S'):
	'''converts a unix time to formatted string (assumes UTC)
	
	:param t: UNIX timestamp (in seconds)
	:param type: float
	:param format: String output format
	:param type: string
	:returns: string represenation of the time
	
	>>> from datetime import datetime
	>>> t_unix = mjd_to_unix(4.9144e9, seconds=True)
	>>> t_unix
	1407683199.9999998
	>>> unix_to_string(t_unix)
	'2014/08/10/15:06:40'
	>>> unix_to_string(t_unix, format='%D')
	'08/10/14'
	'''
	
	try:
		return map(lambda i: datetime.fromtimestamp( i, utc).strftime(format), t)
	except:
		return datetime.fromtimestamp( t, utc).strftime(format)


def getCOFA(ms):
    """
    Return the ITRF coordinates, Longitude and Latitude of the center of the 
    array of the observatory (from the CASA Observatories table) for the 
    specified ms.  - Todd Hunter
    """
    try:
        antTable = ms+'/ANTENNA'
        tb.open(antTable)
    except:
        print "Could not open ANTENNA table: %s" % (antTable)
        return([],[])
    position = tb.getcol('POSITION')
    station = tb.getcol('STATION')
    name = tb.getcol('NAME')
    tb.close()
    try:
        antTable = ms+'/OBSERVATION'
        tb.open(antTable)
        myName = tb.getcell('TELESCOPE_NAME')
        tb.close()
    except:
        print "Could not open OBSERVATION table to get the telescope name: %s" % (antTable)
        myName = ''
        return
    return(getCOFAForObservatory(myName))


def getCOFA(ms):
    """
    Return the ITRF coordinates, Longitude and Latitude of the center of the 
    array of the observatory (from the CASA Observatories table) for the 
    specified ms.  - Todd Hunter's code 
    """
    try:
        antTable = ms+'/ANTENNA'
        tb.open(antTable)
    except:
        print "Could not open ANTENNA table: %s" % (antTable)
        return([],[])
    position = tb.getcol('POSITION')
    station = tb.getcol('STATION')
    name = tb.getcol('NAME')
    tb.close()
    try:
        antTable = ms+'/OBSERVATION'
        tb.open(antTable)
        myName = tb.getcell('TELESCOPE_NAME')
        tb.close()
    except:
        print "Could not open OBSERVATION table to get the telescope name: %s" % (antTable)
        myName = ''
        return
    return(getCOFAForObservatory(myName))

def getCOFAForObservatory(observatory):
    """
    Return the ITRF coordinates, Longitude and Latitude of the center of the 
    array of the specified observatory (from the CASA Observatories table).
    - Todd Hunter
    """
    u = simutil.simutil()
    if (casadef.python_library_directory.find('telcalsa') < 0):
        repotable = os.getenv("CASAPATH").split()[0]+"/data/geodetic/Observatories"
    else: # casapy-telcal does not have CASAPATH defined on sciops
        repotable = casadef.python_library_directory.split('/lib')[0] + "/data/geodetic/Observatories"
    tb.open(repotable)
    Name = tb.getcol('Name')
    myType = ''
    for i in range(len(Name)):
        if (Name[i] == observatory):
            Long = tb.getcell('Long',i)
            Lat = tb.getcell('Lat',i)
            Height = tb.getcell('Height',i)
            myType = tb.getcell('Type',i)
            if (myType == 'ITRF'):
                cx = tb.getcell('X',i)
                cy = tb.getcell('Y',i)
                cz = tb.getcell('Z',i)
            else:
                # WGS84
                output = u.long2xyz(Long*math.pi/180.,Lat*math.pi/180.,Height,myType)
                (cx,cy,cz) = output
            break
    return(cx,cy,cz,Long,Lat)

def getPolmask(meta):
    ''' Read meta data and return 
        the polarization strings

        :param meta: measurement set
        :type meta: dictionary
        

        :returns: list of boolean values for mask
	:rtype: bool list
    '''

    polavg = []
    for pol in meta['pol']:
        if pol in ['XX','YY','RR','LL']:
            polavg.append(False)
        else:
            polavg.append(True) # true to mask

    return polavg


def pwvFromMS(vis, spw=None, antenna=None, time=True, zenith=True):
	''' extracts measured pwv from each antenna
	
	:param vis: measurement set
	:type vis: str
	:param spw: spectral window with WVR measurements (None = the first spw with 4 channels)
	:type spw: str
	:param antenna: list of antennas (None = all antennas)
	:type antenna: int
	:returns: time,pwv
	:rtype: dict
	'''

	t, alt, Tpwv = TpwvFromMS(vis, spw=spw, antenna=antenna, time=True)
	
	if t is not None:
		pwv = {}
		
		for antenna in Tpwv.keys():
				pwv[antenna] = pwvFromTpwv( Tpwv[antenna] , elevation = alt[antenna], zenith=zenith)
	else:
		pwv = None
		
	if time:
		return t,pwv
	else:
		return pwv


def pwvFromTpwv(Tsrc0, site='AOS', zenith = True, elevation = 90., eta_c=0.98, T_loss=275.0):
	''' calculates the PWV from the WVR data in the MS. Optionally,
	the PWV can be corrected to PWV_zenith, using the elevation and
	zenith keywords.
	
	:param Tsrc: measurements of the 4 WVR channels with time. Optionally, this can also be a 4-element array for a single timestep.
	:type Tsrc: [4, nint]-float array
	:param site: antenna site (currently only 'AOS' or 'OSF' are supported)
	:type site: str
	:param zenith: correct towards zenith
	:type zenith: boolean
	:param elevation: source elevation (in degrees)
	:type elevation: float
	:param eta_c: sky coupling constant eta_c
	:type eta_c: float
	:param T_loss: sky coupling constant T_loss
	:type T_loss: float
	:returns: PWV (in mm)
	:rtype: float array
	'''
	
	# make sure we're working with a numpy array here
	Tsrc0 = np.array( Tsrc0 )
	
	# check if it's a [4,nint] numpy array
	# if not, convert it from a 4-element array into a [4,1] numpy array
	try:
		nchan, nint = Tsrc0.shape
	except ValueError:
		Tsrc0 = np.array([[i] for i in Tsrc0])
		nchan, nint = Tsrc0.shape
	
	Tsrc = (Tsrc0 - (1.0-eta_c)*T_loss)/eta_c

	pw=[0.0,0.0,0.0,0.0]

	# sets up fitting constants depending on the site:
	if site == "OSF":   # constants:
		tau0 = np.array([0.024,0.02,0.009,0.01])
		r = np.array([1.205,0.402,0.177,0.116])
		Tphys = 265.0
	elif site == "AOS":  # constants:
		tau0 = np.array([0.027,0.02,0.010,0.01])
		r = np.array([1.193,0.399,0.176,0.115])
		Tphys = 255.0

	# creating matrices from the parameters to allow mathematical operations
	tau0M = np.matrix(tau0).T.dot(np.matrix(np.ones(nint)))
	rM = np.matrix(r).T.dot(np.matrix(np.ones(nint)))
	
	tel = np.clip(1.0-Tsrc/Tphys, 0.02, 1e8)
	wt = 1.0 - ( (tel-0.5)/0.49)**2
	
	pw = np.array(-( np.log(tel) + tau0M ) / rM)

	ws = np.sum(pw*wt, axis=0)
	
	pwv_los = ws/np.sum(wt, axis=0)
	
	if zenith:
		return np.abs(pwv_los*np.sin(np.radians(elevation)))
	else:
		return np.abs(pwv_los)




def TpwvFromMS(vis, spw=None, antenna=None, time=True):
	''' Extracts the time, elevation, and the measurement
	of the 4 WVR channels for each antenna
	from the measurement set and returns a dictionary
	with the data stored as [4, nint]-numpy arrays.
	
	:param vis: measurement set
	:type vis: string
	:param spw: spectral window with the WVR measurements
	            (None = take the first spw with 4 channels)
	:type spw: string
	:param antenna: list with antennas (None = all antennas)
	:type antenna: integer list
	:param time: extract time array?
	:type time: boolean
	:returns: time, elevation, Tsky
	:rtype: dict
	'''
	
	Tpwv = {}
	t = {}
	alt = {}
	
	# get the spw with WVR (must have length 4)
	if spw is None:
		tb.open('%s/SPECTRAL_WINDOW' % vis)
		try:
			spw = '%i' % list(tb.getcol('NUM_CHAN')).index(4)
		except ValueError:
			if time:
				return None, None
			else:
				return None
		
		tb.close()

	# get the fields
	tb.open('%s/FIELD' % vis)
	fields=tb.getcol('NAME')
	nfields=len(fields)
	ref_dir=tb.getcol('REFERENCE_DIR')
	ra=np.zeros(nfields, dtype=np.float)
	de=np.zeros(nfields, dtype=np.float)
	for i in range(nfields):
		ra[i]=np.float(ref_dir[:,0,i][0])
		de[i]=np.float(ref_dir[:,0,i][1])
	tb.close()
	# setting reference frame to ALMA
	me.doframe(me.observatory('ALMA'))
	
	# get the antennas
	if antenna is None:
		tb.open('%s/ANTENNA' % vis)
		ant = tb.getcol('NAME')
		iant = range(len(ant))
		tb.close()
	else:
		iant = antenna
	
	# opens the measurement set
	tb.open(vis)
	
	# loop through all antennas
	first = True
	for i in iant:
		# reduces data to spw and antenna
		tb1 = tb.query('DATA_DESC_ID == %s && ANTENNA1 == %i' % (spw, i))
		
		# extracts the Tpwv and time
		Tpwv[i] = np.array(tb1.getcol('DATA')[0][:][:])
		t[i] = np.array(tb1.getcol('TIME'))
		tdt = unix_to_string(mjd_to_unix(t[i], seconds=True))
		field = tb1.getcol('FIELD_ID')
		elev = []
		for ts,fs in zip(tdt, field):
			me.doframe(me.epoch('UTC', qa.quantity(ts)))
			b=me.direction('j2000', '%.2frad' % ra[fs], '%.2frad' % de[fs])
			elev.append(qa.convert(me.measure(b, 'azel')['m1'],'deg')['value'])
		alt[i] = np.array(elev)

	# close the MS
	tb1.close()
	tb.close()
	
	if time:
		return t, alt, Tpwv
	else:
		return Tpwv


def getTime_2pt(vis,ant1=0,ant2=1,WVR=False,spwnum=None):  
    ''' Read an ms file and return to zero
        starting time in seconds - note these
        are the recored times so there are gaps
        due to sub scans etc

        :param vis: the measurement set
        :type vis: string
        :param ant1: antenna 1 to use (DEFAULT = 0)
        :type ant1: int 
        :param ant2: antenna 2 to use (DEFAULT = 1)
        :type ant2: int 
        :param WVR: if the time is needed for WVR soln, note uses wvr.gcal type soln tables
        :type WVR: bool
        :param spwnum: the SPW number to use (easy to input from meta - or find)
        :type spwnum: integer

        :returns time: the time for the observations
	:rtype: float array
    '''

    
    tb.open(vis)
    if not WVR:
        # select last spw and then query fixed ants 0 and 1
        # should work in all cases - option above to change if not
        if spwnum is None:
            spwnum=getLastSpwID_ms(vis)

        
        tb1 = tb.query("ANTENNA1 == %s && ANTENNA2 == %s && DATA_DESC_ID == %s"%(ant1,ant2,spwnum))
    
        time = tb1.getcol('TIME')

        time = np.array(time) - time[0]

    if WVR:
        if spwnum is None:
            vis_use = vis.split('.')[0]+'.ms'
            spwnum=getLastSpwID_ms(vis_use)

        
        tb1 = tb.query("ANTENNA1 == %s && SPECTRAL_WINDOW_ID == %s"%(ant1,spwnum))
    
        time = tb1.getcol('TIME')

        time = np.array(time) - time[0]
    tb.close()
    return time



def getPhase_2pt(query,data,polmask,WVR,pols,Flags=True):  
    ''' Section of 2-pt calculation to 
        get the polarization averaged phases
        can also pull the WVR per antenna data

        :param query: preset query to get data from file
        :type query: 
        :param data: type of data to be read in
        :type data: string
        :param polmask: mask of the polarizations to average
        :type polmask: list bool
        :param WVR: if data is from WVR dataset or not
        :type WVR: bool
        :param pols: all polarization of the data
        :type pols: list string
        
        :returns: the phases of baseline or ant dependent on input query
	:rtype: float array
    '''

    if not WVR:
        
        com_vis =  query.getcol(data)
        

	# run the old way to test the plots due to error now??
	poldata = np.array(np.zeros_like(com_vis), dtype=np.bool)
	

	phase=[]
	for ii in range(len(com_vis)):
		if polmask[ii] == False:
                    #old way                    
                    #phase.append(np.ma.masked_array(com_vis[ii], flagdata[ii]))

                    # new way 
                    phase_app = []
                    if Flags:
                        phase_hld = com_vis[ii]
                        com_flag=query.getcol('FLAG')
                        flags_use = com_flag[ii] # selects correct polarisation
                        # loop the phases 
                        phase_hld = [val if not flags_use[0][id_u] else np.nan for id_u,val in enumerate(phase_hld[0])]
                        phase_app.append(phase_hld)
                        phase.append(phase_app) # get to same structure of data
                       
                    else:
                        phase.append(com_vis[ii])
                        

        phase = np.angle(phase)


    if WVR:
        if Flags:

            com_vis =  query.getcol(data)[0][0]
            flags_use = query.getcol('FLAG')[0][0]
            phase =  [val if not flags_use[id_u] else np.nan for id_u,val in enumerate(com_vis) ]

	# this is fine for wvr stream itself, it is only one array so np.mean done not matter, but this returns a single array
        else:
            phase= query.getcol(data)[0][0]
            
        phase = np.angle(phase)
        
        
    return phase   






def outlier_remove(phases,factor=3.0):  
    ''' this reads in either a single phase array or more complex data structure
        such as the raw phase we read from EOC data, i.e. both pols

        :param phases: array single or complex
        :type phases: float array
	:param factor: multiples of the standard deviation the mean and data differ by, default = 3.0
        :type factor: float
        
        
        :returns: the phases in same format as the input but with outliers as nans - i.e wont plot or be used in unwraps later
	:rtype: float array
	'''

    # get the average standard deviation of the array or one pol
    std_check_hold = []
    if len(phases) == 2 or len(phases)==4:
	    for i in range(len(phases[0][0])-10):
		    std_check_hold.append(np.std(phases[0][0][i:i+10]))

	    std_check = np.mean(std_check_hold)

    else:
	    for i in range(len(phases)-10):
		    std_check_hold.append(np.std(phases[i:i+10]))
		    
	    std_check = np.mean(std_check_hold)


    # now do the outlier set to nan and reinitilise the array for output
    if len(phases) == 2 or len(phases)==4:
	    phases_trim = phases.copy()
	    for no in range(len(phases)):
		    # so now we can do the index of the pol in a loop
		    for i in range(len(phases[no][0])-10):
			    if i < 5:
				    if abs(phases[no][0][i] - phases[no][0][0]) > factor*std_check:
					    phases_trim[no][0][i]=np.nan

			    else:
				    if abs(phases[no][0][i] - np.mean(phases[no][0][i-5:i+5])) > factor*std_check:
					    phases_trim[no][0][i]=np.nan

    else:
	    phases_trim = phases.copy()
	    for i in range(len(phases)-10):
		    if i < 5:
			    if abs(phases[i] - phases[0]) > factor*std_check:
				    phases_trim[i]=np.nan

		    else:
			    if abs(phases[i] - np.mean(phases[i-5:i+5])) > factor*std_check:
				    phases_trim[i]=np.nan


    
    
    return phases_trim


def wrapCheckTimeRMS(phase, time, lim=90, over=60, std_val=3.5, auto=False, rapid=False):
    ''' Check the phase for incorrect wraps

    :param phase: phase in radians
    :type phase: float array
    :param time: the time samples
    :type time: float array
    :param lim: limit for consecutive phases to vary by (180 - lim)
    :type lim: float
    :param over: number of elements to use for SD calc
    :type over: integer
    :param std_val: value the phase SD must be above to confirm a jump
    :type std_val: float
    :param auto: to calculate the limit automatically based on the data SD over 60 seconds
    :type auto: boolean
    :param rapid: compensates for rapidly fluctuating data - jump set as 30
    :type rapid: boolean
    :returns: phases after check - corrected 
    :rtype: float array
    

    If no limit is set the default of 90 degrees is used
    to find possible jumps, i.e. incorrect wraps are found if the
    phase is 180 deg (pi) - lim = 90 deg. Given
    the phase should have been unwrapped there should
    not be large differences between the consecutive phase points.
    Time checks are also performed as phase differences over a
    scan gap can mean incorrect wraps added back if large lim
    values are used, i.e. to detect jumps ~ 120 deg in size

    The limit sets the sensitivity to detect the jump.
    Larger lim value means more jumps found, i.e lots of consecutive
    phases vary by 180 - lim, which is small. To check then
    we make sure the jump is not over a time gap
    then check that the rolling means over/2 before and
    after the are different by the 2 x the nominal standard
    deviation x std_val (input). This acts as the robustness
    to ensure the jump found is real or not. To be 
    robust to the places where phases are noisy in a row the
    rolling means are shifted when a jump is correctly
    found, such that a subsequent jump close in time
    checks the adjusted means which, would be aligned if only 
    one jump is real and rest were noise (for clustered points).

    Note, limitation is the before rms is used if enough 
    elements to do so, otherwise uses the starting 
    phase values. Also has option rapid, this means if you
    see rapid changes in the phases sometimes over the gaps
    it will think there is a jump, i.e. more than 1.3 * pi in
    a 60 sec gap. If you use option rapid, then it compares the
    same jump but over 30 seconds, only if jump is larger than
    1.3 * pi over a 30 sec gap in the 60 second averaged data does it
    perform a wrap correction 
    '''

    # to deal with nans we need continous array so cut down
    # this will not be used in the scaling code 
    
    phase_cp = phase.copy() # to keep the original
    phase = phase[np.isfinite(phase)]
    time_cp = time.copy()
    time = time[np.isfinite(phase)]
	    

    # set up the averaging to calc SD
    diff_elems = np.mean(np.diff(time_cp))

    # this is now the number of elements to loop over given the over time in SECONDS (i.e. input in sec)
    over_elems = np.int(np.fix(over/diff_elems))   
    
    #print over
    std_hold =[]
    mean_hold =[]
    for i in range(len(phase)-over_elems):
        # this creates a 10 element SD values (gap/nan resilient)
        std_hold.append(np.std(phase[i:i+over_elems]))
        mean_hold.append(np.mean(phase[i:i+over_elems]))
    
    std_mean = np.mean(std_hold)


    # auto true to set lim
    if auto == True:
	    lim = std_hold[0]  
    
    # check phases within the limits (similar to unwrap code)
    diff = np.diff(phase)
    pup    = np.cumsum(diff < -(np.pi - np.radians(lim)))
    pdown  = np.cumsum(diff > (np.pi - np.radians(lim)))


    # finds 0 1 -1 to say where a jump is
    diff_ps = np.diff (pup - pdown)
    
    # now find the index where a jump may have been 
    # found and that the SD is larger than expected

    for i in range(len(phase)-over_elems-3):  # -1 for index and 2 extra for i+2 for mean hold
                                              # must be phase - over_elems - 1 - 2
      
        # if positive and not above std val minus one from pup array
        # after the i+1 index (pup, pdown offset from diff_ps by 1)
        # if negative then take one off the pdown array (inc offset)
        if diff_ps[i] == 1:
            # means a down break is detected so add one
            # to the pup to MOVE UP
            
            if i-over_elems+2 > 0: # does only +ve means start jumps in the else
                
                ## could change this to absolute? Calc a previous mean and after - or shift the mean
                ## check that that is in a range??

                if std_mean*2*std_val > abs(mean_hold[i - over_elems + 2] - mean_hold[i + 2]): 
                    pup[i+1:] = pup[i+1:] - 1  
                    #print 'FALSE JUMP' 
                    
                elif time[i+2]-time[i+1] > 3.5:
                    # this is the gap check
                    
                    pup[i+1:] = pup[i+1:] - 1  
                    #print 'FALSE JUMP' 
                    
                else:
                    # change the previous mean by 360
                    # such that after a jump is the previous data
                    # is corrected so where
                    # there is noisy data it doesnt find another jump due
                    # to the mean check - shift wrong way as its running on previous to now (so backwards)
		    mean_hold[0:i - over_elems/2 + 2] = np.array(mean_hold[0:i - over_elems/2 + 2]) - 2*np.pi
            

	    else:
                if std_mean*2*std_val > abs(phase[i+1] - mean_hold[i + 2]):
                    pup[i+1:] = pup[i+1:] - 1  
                    #print 'FALSE JUMP' 
                    
                elif time[i+2]-time[i+1] > 3.5:
                    # this is the gap check
                    
                    pup[i+1:] = pup[i+1:] - 1  
                    #print 'FALSE JUMP' 
                    

            


        if diff_ps[i] == -1:
            # means a upward break is detected so it
            # adds one to the pdown to MORE DOWN
            # so pup-pdown is -1
            
            if i-over_elems+2 > 0:
                #print 'YES'
                if std_mean*2*std_val  > abs(mean_hold[i - over_elems + 2] - mean_hold[i + 2]):

                    pdown[i+1:] = pdown[i+1:] - 1  
                    #print 'FALSE JUMP'
                    
                elif time[i+2]-time[i+1] > 3.5:
                    pdown[i+1:] = pdown[i+1:] - 1  
                    #print 'FALSE JUMP'
                    
                else:
                    
                    mean_hold[0:i - over_elems/2 + 2] = np.array(mean_hold[0:i - over_elems/2 + 2]) + 2*np.pi
            else:
                if std_mean*2*std_val  > abs(phase[i+1] - mean_hold[i + 2]):

                    pdown[i+1:] = pdown[i+1:] - 1  
                    #print 'FALSE JUMP'
                    
                elif time[i+2]-time[i+1] > 3.5:
                    pdown[i+1:] = pdown[i+1:] - 1  
                    #print 'FALSE JUMP'
                    



    ph = np.zeros_like(phase_cp)
    ph[0] = phase_cp[0]  # hopefully start is not nan
    

    lp = 1
    cr = 0
    correct = pup-pdown
    while lp < len(phase_cp):
        # now correct on the fly with the 
        # first element of pup-pdown
        # then remove the element
        # if the main array is not a nan and correction was applied
        # have to do this way as the output is a phase
        # stream to be averaged with another 
        
        if np.isfinite(phase_cp[lp]): 
            ph[lp] = correct[cr] * 2*np.pi + phase_cp[lp]
            cr=cr+1
        else:
            # i.e. no correction as phase is a nan - so just fill ph with a nan
            ph[lp] = phase_cp[lp]

        lp=lp+1
   
    ##return ph

    ## NOW THE RMS CHECKING ## - note may fail on nans !!

    # use the phase here and do the means and RMS loop checking
    # always use 60 as the least average

    ph_m = np.ma.masked_array(ph,np.isnan(ph))
    over_elems = np.int(np.fix(60 /diff_elems))
    over = np.int(np.fix(60 / diff_elems))
    mean_hold =[]
    time_hold = []
    for i in range(len(ph_m)-over_elems):
        mean_hold.append(np.mean(ph_m[i:i+over_elems]))
	time_hold.append(np.mean(time_cp[i:i+over_elems]))  

    # populate the difference to pad to length and keep track
    jump = np.zeros_like(time_cp)
    jump[0:over/2] = mean_hold[0]
    for jj in range(len(mean_hold)):
        jump[over/2+jj] = mean_hold[jj] 
    jump[len(mean_hold)+over/2:] = mean_hold[len(mean_hold)-1]

    pup = np.zeros_like(time_cp)
    pdown = np.zeros_like(time_cp)

    # so this is the re-check based on the rolling RMS
    kk = 0
    if rapid == True:
	    over = np.int(np.fix(30 / diff_elems))

    while kk < len(mean_hold)-over:   ## based on elements in array
    #print time_hold[kk]
	over_use = over
	if time_cp[kk+over_use] - time_cp[kk] > over:   
        ## time not continous always - therefore larger gaps
		         
        ## so time gap is larger than 60 so need to account for this as phase may change considerably
        ## find new over_use to make gap 60
        ## only happens if gaps - note if large gap > 60 this may fail..
            while over_use > 1.:
                if time_cp[kk+over_use] - time_cp[kk] < over:
                    break
                over_use = over_use - 1
            #print over_use
        ## if break was large it will hit zero as time gap will always be >60 
        if jump[kk] > jump[kk+over_use]+1.3*np.pi:


        # set the pup from the time realted to time hold, usually in a gap
        # so find time index in real array

            storin = [index for index,value in enumerate(time_cp) if value > time_hold[kk]]
        
	    if abs(time_cp[storin[0]]-time_cp[storin[0]-3]) > 5.0:#  and abs(ph[storin[0]+3]-ph[storin[0]-3]) > np.pi: 
                 
                pup[storin[0]:] = pup[storin[0]:]+1
            #pup[kk+over:]=pup[kk+over:]+1
        
	    #print time1[kk+over/2]
	    #print time_hold[kk]
		##print time_cp[kk]
	    #print np.degrees(jump[kk]),np.degrees(jump[kk+over])
        
	        jump[:storin[0]]=jump[:storin[0]] - 2*np.pi  # if we say a jump correct previous to avoid double correct

            ##jump[:kk+over]=jump[:kk+over] - 2*np.pi  # if we say a jump correct previous to avoid double correct

	    #print 'going up'
            ##kk=kk+over # find a jump then move over length on
	        kk= storin[0]+over_use
	    else: 
		kk=kk+5
        

        if  jump[kk] < jump[kk+over_use] - 1.3*np.pi:

	    storin = [index for index,value in enumerate(time_cp) if value > time_hold[kk]]
        
	    if abs(time_cp[storin[0]]-time_cp[storin[0]-3]) > 5.0:#  and abs(ph[storin[0]+3]-ph[storin[0]-3]) > np.pi:     
	        pdown[storin[0]:] = pdown[storin[0]:]+1
           
	        jump[:storin[0]]=jump[:storin[0]] + 2*np.pi # ie forward is down, prev is up


	        kk = storin[0]+over_use
	    else:
		kk=kk+5
        

        else:
            kk=kk+1
            #print 'No more jump'

    
    ph_final = np.zeros_like(ph)
    ph_final[0:] = ph[0:] + (pup - pdown) *2*np.pi



    return ph_final

def toPathlength(freq, phase, radians = True, verbose = True):

    ''' To read in the rest frequency and the allan
    variance values (or any type of phase) and to
    convert these into a standard path length
    in microns. Options for radians or degrees input
    default is radians

    Includes a crude check to see if frequency is in GHz or not

    :param freq: frequency value, Hz or GHz only
    :type freq: float
    :param phase: the input phases
    :type phase: float/double array
    :param radians: select input unit in radians, if not degrees
    :type radians: bool
    :param verbose: returns a print if true
    :type verbose: bool

    :returns: convesion to microns the path length from phase
    :rtype: float array

    '''

    if verbose: print " Now converting all the Phases to Path Length Delays"

    if freq/1000. < 1:
        # the units are in GHz change to Hz
        freq = freq*1.e9


    if radians:
        Pathlength = 1.e6*constants.c*np.array(phase)/(2*np.pi*freq)

    else:
        Pathlength = 1.e6*constants.c*np.array(np.radians(phase))/(2*np.pi*freq)


    return Pathlength
        
def getSTDall(data):
	'''Returns the rms i.e. the standard deviation of a data array for SSF plots
	
	:param float data: dictionary with phases before ('DATA') and after ('CORRECTED_DATA') WVR correction.
	
	:rtype: dict
	
	'''
	print " Calculating RMS"
	rmsdata={'name':[],'phase_rms':[],'corr_rms':[]}
	for key in data['DATA']:
		# append the name
		rmsdata['name'].append(key)

		# do uncorrected data 
		p  = data['DATA'][key][:,1] # phase
		t  = data['DATA'][key][:,0] # time

		

		# calculate rms
		r = STDall(p)
		rmsdata['phase_rms'].append(r)

		# now do corrected data
		cp = data['CORRECTED_DATA'][key][:,1] # corr phase
		ct  = data['CORRECTED_DATA'][key][:,0] # time
		
		# calculate rms
		cr = STDall(cp)
		rmsdata['corr_rms'].append(cr)
	print " * Done Calculating RMS"
	return rmsdata


def STDall(phase):
    '''Calc standard deviation over time and return the average
    deals with nan values by excluding these values from the
    calculation of the STD

    :param phase: any unwrapped input phase
    :type phase: float array
    :returns: Stanard deviation for independent 'over' cycles
    :rtype: float

    Read a phase array which must be unwrapped
    and have been checked for incorrect jumps
    (although this should not significantly affect
    the statistic is jumps are minimal) and
    calculated the STD accounting for nan data.
    Note this is STANDARD DEVIATION HERE - takes out the
    mean value. RMS with mean or fit removed SHOULD
    be the same value for zero centred phase.
    '''

    # set up the averaging to calc standard deviation in this case
    

    phase_use = phase[np.isfinite(phase)]
    std_mean = np.std(phase_use)

    return std_mean


def STDC(phase,over):
    '''Calc STD over time and return the average
    deals with nan values by excluding these values from the
    calculation of the STD

    :param phase: any unwrapped input phase
    :type phase: float array
    :param over: index to include in one calc (i.e. time range) - [limit len(phase)-1]
    :type over: int 
    :returns: Stanard deviation for independent 'over' cycles
    :rtype: float

    Read a phase array which must be unwrapped
    and have been checked for incorrect jumps
    (although this should not significantly affect
    the statistic is jumps are minimal) and
    calculated the STD in overlapping times (user input)
    only for this EOC data with 0.96 integration times.
    Note this is STANDARD DEVIATION HERE - takes out the
    mean value. RMS with mean or fit removed SHOULD
    be the same value for zero centred phase.
    '''

    # set up the averaging to calc standard deviation in this case
    
    std_hold =[]
    for i in range(len(phase)-over):
        ph_hold = phase[i:i+over]
        phase_use = ph_hold[np.isfinite(ph_hold)]
        std_hold.append(np.std(phase_use))
        #if i == 0:
        #    print std_hold
    
    std_mean = np.mean(std_hold)

    return std_mean



def getRMSall(data):
	'''Returns the rms i.e. the RMS of a data array for SSF plots
           the RMS is equal to the STD IF and only the linear term is removed
           hence the MEAN of the data is ZERO - if the data is offset from ZERO
           AND you want to account for this in the noise - then use the RMS 
	
	:param float data: dictionary with phases before ('DATA') and after ('CORRECTED_DATA') WVR correction.
	
	:rtype: dict
	
	'''
	print " Calculating RMS"
	rmsdata={'name':[],'phase_rms':[],'corr_rms':[]}
	for key in data['DATA']:
		# append the name
		rmsdata['name'].append(key)

		# do uncorrected data 
		p  = data['DATA'][key][:,1] # phase
		t  = data['DATA'][key][:,0] # time

		

		# calculate rms
		r = RMSall(p)
		rmsdata['phase_rms'].append(r)

		# now do corrected data
		cp = data['CORRECTED_DATA'][key][:,1] # corr phase
		ct  = data['CORRECTED_DATA'][key][:,0] # time
		
		# calculate rms
		cr = RMSall(cp)
		rmsdata['corr_rms'].append(cr)
	print " * Done Calculating RMS"
	return rmsdata


def RMSall(phase, linear=False):
    '''Calc RMS (root mean square) over time and return the average
    deals with nan values by excluding these values from the
    calculation - note the RMS is equal to the STD IF and 
    only when the linear term is removed, hence the MEAN of the data is 
    ZERO - if the data is offset from ZERO AND you want to account 
    for this in the noise - then use the RMS 

    :param phase: any unwrapped input phase
    :type phase: float array
    :returns: Stanard deviation for independent 'over' cycles
    :rtype: float

    Read a phase array which must be unwrapped
    and have been checked for incorrect jumps
    (although this should not significantly affect
    the statistic is jumps are minimal) and
    calculated the RMS accounting for nan data.
    RMS with mean or fit removed SHOULD
    be the same value for zero centred phase (i.e an STD),
    if not the RMS will be higher as also accounts for any offset.
    '''

    # set up the averaging to calc standard deviation in this case
    
    rms_phase = phase.copy()  ## copy for reducndancy
    if linear:
	    # this should be same as the STD as we take the linear mean out
	    phase_use = rms_phase[np.isfinite(rms_phase)] - np.mean(rms_phase[np.isfinite(rms_phase)])
    else:
	    phase_use = rms_phase[np.isfinite(rms_phase)]
    rms_mean = np.sqrt(np.mean(phase_use**2))

    return rms_mean


def RMSC(phase,over):
    '''Calc RMS over time and return the average
    deals with nan values by excluding these values from the
    calculation 

    :param phase: any unwrapped input phase
    :type phase: float array
    :param over: index to include in one calc (i.e. time range) - [limit len(phase)-1]
    :type over: int 
    :returns: Stanard deviation for independent 'over' cycles
    :rtype: float

    Read a phase array which must be unwrapped
    and have been checked for incorrect jumps
    (although this should not significantly affect
    the statistic is jumps are minimal) and
    calculated the RMS in overlapping times (user input)
   
    Note this is RMS HERE
    Because the data stream is assumed LONGER than the 
    chosen averaging time for statistics one must
    offet the general stream to zero phase to
    make 2 independent runs for with and without WVR scaling
    applied consistent - otherwise the offset can be
    vastly differernt and constitutes to more to the RMS.

    Although calibration will do this 'more' correctly,
    the fastest estimate is to remove the mean from the
    entire phase stream before processing 
    '''

    rms_phase = phase.copy()  ## copy for reducndancy

    # set up the averaging to calc standard deviation in this case
    
    rms_phase = rms_phase - np.mean(rms_phase)  # offset total solution to zero phase
    # then one can test varaibility of the phase stream without taking 
    # out short term fluctuations

    rms_hold =[]
    for i in range(len(rms_phase)-over):
        ph_hold = rms_phase[i:i+over]
        phase_use = ph_hold[np.isfinite(ph_hold)]
        # append all RMS values from the time t to t+over
        rms_hold.append(np.sqrt(np.mean(phase_use**2)))
        
    
    rms_mean = np.mean(rms_hold)

    return rms_mean
